MODULE attenuation
CONTAINS

!==============================================================================!
!
!  ATTENUATION
!
!  This module contains routines used to implement attenuation.
!  The method is based after the paper:
!  'Efficient Modeling of Q for 3D Numerical Simulations of Wave Propagation'
!   by, Pengcheng Liu and Ralph J. Archuleta, BSSA, Vol. 96, No. 4A,
!   pp. 1352-1358, August 2006, doi: 10.1785/0120050173
!
!   Michael Thorne
!   Last Updated: 7/24/07
!
!==============================================================================!

!==============================================================================!
!
!  Q_INIT
!
!  Initialize variables for attenuation
!
!==============================================================================!
SUBROUTINE Q_init
USE global, ONLY: nx, nz, rlxt, wtr1, wtr2, Mu_1, Mu_2, Nk, mvr1, mvr2
IMPLICIT NONE
REAL, DIMENSION(nx,nz) :: rtemp

!Set relaxation times
!-------------------------------------
rlxt =(/.00172333,.00180701,.00538887,.0199322, &
     &  .08498330,.40933500,2.0595100,13.26290/) 

!Allocate Variables
!-------------------------------------
CALL allocate_Q
wtr1 = 0.0
wtr2 = 0.0

!Calculate unrelaxed modulus
!-------------------------------------
CALL unrelaxed_modulus(Mu_1,Nk,1)   
CALL unrelaxed_modulus(Mu_2,Nk,2)

!Calculate weight functions
!-------------------------------------
CALL weight_coeff(rtemp,1,1)
wtr1(1:nx,1:nz,1) = rtemp(1:nx,1:nz)
CALL weight_coeff(rtemp,1,2)
wtr2(1:nx,1:nz,1) = rtemp(1:nx,1:nz)

IF (Nk >= 2) THEN
  CALL weight_coeff(rtemp,2,1)
  wtr1(1:nx,1:nz,2) = rtemp(1:nx,1:nz)
  CALL weight_coeff(rtemp,2,2)
  wtr2(1:nx,1:nz,2) = rtemp(1:nx,1:nz)
ENDIF
IF (Nk >= 3) THEN
  CALL weight_coeff(rtemp,3,1)
  wtr1(1:nx,1:nz,3) = rtemp(1:nx,1:nz)
  CALL weight_coeff(rtemp,3,2)
  wtr2(1:nx,1:nz,3) = rtemp(1:nx,1:nz)
ENDIF
IF (Nk >= 4) THEN
  CALL weight_coeff(rtemp,4,1)
  wtr1(1:nx,1:nz,4) = rtemp(1:nx,1:nz)
  CALL weight_coeff(rtemp,4,2)
  wtr2(1:nx,1:nz,4) = rtemp(1:nx,1:nz)
ENDIF
IF (Nk >= 5) THEN
  CALL weight_coeff(rtemp,5,1)
  wtr1(1:nx,1:nz,5) = rtemp(1:nx,1:nz)
  CALL weight_coeff(rtemp,5,2)
  wtr2(1:nx,1:nz,5) = rtemp(1:nx,1:nz)
ENDIF
IF (Nk >= 6) THEN
  CALL weight_coeff(rtemp,6,1)
  wtr1(1:nx,1:nz,6) = rtemp(1:nx,1:nz)
  CALL weight_coeff(rtemp,6,2)
  wtr2(1:nx,1:nz,6) = rtemp(1:nx,1:nz)
ENDIF
IF (Nk >= 7) THEN
  CALL weight_coeff(rtemp,7,1)
  wtr1(1:nx,1:nz,7) = rtemp(1:nx,1:nz)
  CALL weight_coeff(rtemp,7,2)
  wtr2(1:nx,1:nz,7) = rtemp(1:nx,1:nz)
ENDIF
IF (Nk == 8) THEN
  CALL weight_coeff(rtemp,8,1)
  wtr1(1:nx,1:nz,8) = rtemp(1:nx,1:nz)
  CALL weight_coeff(rtemp,8,2)
  wtr2(1:nx,1:nz,8) = rtemp(1:nx,1:nz)
ENDIF

!Initialize Memory Variables
!-------------------------------------
mvr1 = 0.0
mvr2 = 0.0

END SUBROUTINE Q_init
!==============================================================================!

!==============================================================================!
!
! UNRELAXED_MODULUS
! Returns the unrelaxed modulus 
!
! Mod_U:  (nx,ny) - Unrelaxed Modulus
!    Nk:  total number of weight coefficients used
!   nmu:  which grid (1=r1, 2=r2)  
!
!==============================================================================!
SUBROUTINE unrelaxed_modulus(Mod_U,Nk,nmu)
USE global, ONLY: nx, nz, mu1, mu2, rlxt
IMPLICIT NONE
REAL, DIMENSION(nx,nz), INTENT(OUT) :: Mod_U
REAL, DIMENSION(nx,nz) :: w_k
REAL, PARAMETER :: w_r=6.283185308    !1 Hz (2*pi)
REAL :: tau_k
COMPLEX, DIMENSION(nx,nz) :: numer, temp
COMPLEX :: denom, ones
INTEGER, INTENT(IN) :: Nk, nmu
INTEGER :: J

ones = (1.0,0.0)
temp = (0.0,0.0)

DO J=1,Nk
  tau_k = rlxt(J)
  CALL weight_coeff(w_k,J,nmu)
  numer = CMPLX(w_k)
  denom = CMPLX(1.0,w_r*tau_k) ! Gunnar: added CMPLX before the brackets
  temp = temp + numer/denom
ENDDO

temp = ones - temp
Mod_U = ABS(temp)

IF (nmu == 1) THEN
  Mod_U = mu1/Mod_U
ELSEIF (nmu == 2) THEN
  Mod_U = mu2/Mod_U
ENDIF

END SUBROUTINE unrelaxed_modulus
!==============================================================================!


!==============================================================================!
!
! WEIGHT_COEFF
! Returns weight coefficients for the memory variables
!     w:  (nx,nz)  holds the output weight factors
!     k:  return the 'kth' weight coefficient
!   nmu:  which grid (1=r1, 2=r2)  
!
!==============================================================================!
SUBROUTINE weight_coeff(w_k,k,nmu)
USE global, ONLY: nx,nz,Q1,Q2
IMPLICIT NONE
REAL, DIMENSION(nx,nz), INTENT(OUT) :: w_k
REAL, DIMENSION(nx,nz) :: Q
REAL, DIMENSION(8) :: alpha_k, beta_k
INTEGER, INTENT(IN) :: k, nmu


!Initialize Alpha(k) and Beta(k)
!---------------------------------------------------------------
alpha_k =(/.0166958,.03816440,.00984666,-.0136803, &
        & -.0285125,-.0537309,-.0665035,-.133696/)
beta_k  =(/.0898758,.0684635,.0967052,.120172, &
        &  .1307280,.1387460,.1407050,.214647/)


!Fill  Q values from Q-model
!---------------------------------------------------------------
IF (nmu == 1) THEN
  Q = Q1
ELSEIF (nmu == 2) THEN
  Q = Q2
ENDIF

!Convert from Q to Chi
!---------------------------------------------------------------
Q = (3.071 + 1.433*(Q**(-1.158))*log(Q/5.0))/(1.0 + 0.415*Q)

w_k = Q*(Q*alpha_k(k) + beta_k(k))

END SUBROUTINE weight_coeff
!==============================================================================!



!==============================================================================!
!
! QPREM
! Returns Quality factors from the PREM model
!     Q: (nx,nz)  quality factors
!   nmu:  which grid (1=r1, 2=r2)  
!
!==============================================================================!
SUBROUTINE Qprem(Q,nmu) 
USE global, ONLY: nx,nz, r1, r2
IMPLICIT NONE
REAL, DIMENSION(nx,nz), INTENT(OUT) :: Q
REAL :: re, depth, x, dist, Qu 
INTEGER, INTENT(IN) :: nmu 
INTEGER :: I

re = 6371000.0

    DO i=1,nz
      IF (nmu == 1) THEN
        depth=(re-r1(1,i))/1000.
      ELSEIF (nmu == 2) THEN
        depth=(re-r2(1,i))/1000.
      ENDIF

      dist=re/1000.-depth
      x=dist/(re/1000.)

      IF (dist >= 6291.0) THEN
        Qu = 600.0
      ELSEIF (dist < 6291.0 .AND. dist >= 6151.0) THEN
        Qu = 80.0
      ELSEIF (dist < 6151.0 .AND. dist >= 5701.0) THEN
        Qu = 143.0
      ELSEIF (dist < 5701.0 .AND. dist >= 3480.0) THEN
        Qu = 312.0
      ELSEIF (dist < 3480.0) THEN
        Qu = 5000.0
      ENDIF

      Q(:,i) = Qu
    ENDDO

END SUBROUTINE Qprem
!==============================================================================!

!==============================================================================!
!
! ALLOCATE_Q
! Allocate memory variables and unrelaxed modulus
!
!==============================================================================!
SUBROUTINE allocate_Q
USE global, ONLY: nx, nz, Nk, Mu_1, Mu_2, wtr1, wtr2, mvr1, mvr2
IMPLICIT NONE

ALLOCATE(Mu_1(nx,nz))
ALLOCATE(Mu_2(nx,nz))

ALLOCATE(wtr1(nx,nz,Nk))
ALLOCATE(wtr2(nx,nz,Nk))

ALLOCATE(mvr1(nx,nz,Nk))
ALLOCATE(mvr2(nx,nz,Nk))

END SUBROUTINE allocate_Q
!==============================================================================!


END MODULE attenuation
