! Module: my_cart2D.f90
!
! Contains my cart2D MPI library.
!----------------------------------------------------------------------
! Library Subroutines:
!
!    my_cart_init()            sets up 2D MPI topology
!    my_mpi_check()            checks number of invoked processes
!    mpi_communicate()         does exchange of array boundaries
!
!    my_glob2loc(loc,glob,dim) maps local array index to global index
!    my_loc2glob(glob,loc,dim) maps global array index to local index
!----------------------------------------------------------------------
! Library Functions:
!
!    Functions similar to my_glob2loc() and my_loc2glob():
!    local(glob,dim)     reads local array index, returns global index
!    glob(loc,dim)       reads global array index, returns local index
!
!   my_index(glob,dim)        checks if global index resides in local
!                              array. returns 1 or 0.
!----------------------------------------------------------------------
! Interface variables:
!
!   mpi_rank               actual rank number
!   mpi_size               total number of ranks
!   dim_size(mpi_ndims)    number of ranks in each dimension
!   mpi_upper,mpi_lower    rank no of upper/lower neighbour
!   mpi_left,mpi_right     rank no of left/right neighbour
!   mpi_xrank,mpi_zrank    rank no in x- and y-direction ("2D rank-no")
!   mpi_xoffs,mpi_zoffs    offset of first array index in x- and z-dir.
!   glob_xsize,glob_zsize  total size of the global data array
!
! New: Library has Version number, starting with: V 0.01
!----------------------------------------------------------------------
!
! * Version 0.01, 16.05.2001.
!
!     Subroutines in library:
!        my_cart_init(), my_mpi_check(), mpi_communicate()
!        my_glob2loc(), my_loc2glob()
!
!     Interface Variables in library:
!        mpi_rank,  mpi_size,  dim_size(:),
!        mpi_upper, mpi_lower, mpi_left,  mpi_right
!        mpi_xrank, mpi_zrank, mpi_xoffs, mpi_zoffs
!
!   Comment: First and second array index are now called 'x'-index and
!            'z'-index, since the second index is often along depth axis.
!            The occurence of 'y-index' or any variables containing a 'y'
!            for second index description is hopefully removed everywhere.
!----------------------------------------------------------------------
!
! * Version 0.02, 29/30.05.2001:
!
!    New symbols:
!       MPI_X, MPI_Z for array index description, (former: plain
!         numbers ("1"'s or "2"'s) ) used e.g. for array dim_size(:),
!         subroutine my_mpi_check() ...
!
!    New Interface Variables:
!       glob_xsize, glob_zsize : total size of the global data array,
!         note that buffers used for mpi communication don't contribute
!         whereas unused buffers at the edges of the topology contribute
!         to the global array size.
!
!    New Functions:
!      local(glob,dim) and glob(loc,dim) added. Purpose is similar to
!        subroutines glob2loc() and loc2glob() which may now be obsolete.
!
!    Comment:
!      The combination of function names local() and glob() are somewhat
!      unhandy but 'loc' seems to be an implicit routine and 'global'
!      is used also by F90.
!    Question:
!      Are the subroutines my_glob2loc() and my_loc2glob() now obsolete,
!      since the functions local() and glob() exist?
!
!
! (Pri)   Todo:
!----------------------------------------------------------------------------------
!   1     * clear source tree and description (cvs?)
!   1     * check local() and glob() even with odd parameters
!
!         * add variables glob_xsize, glob_zsize for model size
!
!         * subroutine local_size(rank,dim)
!         * subroutine my_position() that works with [km] instead of grid points
!         * subroutine grid2pos() conversion grid index -> location [km]
!         * subroutine pos2grid() conversion location [km] -> grid index
!
!         * check stuff at hitachi
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

module mod_cart2D
  implicit none
  include "mpif.h"
  integer, parameter :: MPI_X=1, MPI_Z=2         ! symbols for index numbering, x=1, z=2

  integer, parameter :: mpi_ndims=2              ! No. of dims in topology, this module is for 2 dims.
  integer, parameter :: mpi_reorder=1            ! flag allowing reordering of ranks
  integer, parameter :: mpi_periods(mpi_ndims)=0 ! topology is non-periodic, first/last rank are no neighbours

  integer mpi_rank,mpi_size       ! actual rank no. and total number of processes
  integer dim_size(mpi_ndims)     ! number of procs for each dimension, set this in main program

  integer mpi_upper,mpi_lower     ! rank numbers of the four neighbour processes
  integer mpi_left,mpi_right

  integer mpi_xrank,mpi_zrank     ! rank positions in 2D topology (X/Z rank coordinate)
  integer mpi_xoffs,mpi_zoffs     ! offset of first local grid point with respect to the global grid
  integer glob_xsize,glob_zsize   ! size of global grid

  integer new_comm                ! new communicator for cartesian topology
  integer mpi_ierr                ! variable for mpi error messages

  contains !##########################################################

  subroutine my_cart_init
    use global ! use mod_params
    implicit none
    integer, parameter :: FIRST_INDEX=0,SECOND_INDEX=1 ! only used here, for MPI_Cart_shift()
    integer, parameter :: NEXT_NEIGHBOUR=1             ! distance for MPI_Cart_shift() is usually 1

    call MPI_Cart_create(MPI_COMM_WORLD,mpi_ndims,dim_size,mpi_periods,mpi_reorder,new_comm,mpi_ierr)
    ! read size and actual rank in new topology (new_comm)
    !
    call MPI_Comm_size(new_comm,mpi_size,mpi_ierr)
    call MPI_Comm_rank(new_comm,mpi_rank,mpi_ierr)

    ! find neighbours
    !
    call MPI_Cart_shift(new_comm,FIRST_INDEX, NEXT_NEIGHBOUR,mpi_left, mpi_right,mpi_ierr)
    call MPI_Cart_shift(new_comm,SECOND_INDEX,NEXT_NEIGHBOUR,mpi_upper,mpi_lower,mpi_ierr)

    mpi_xrank= mpi_rank/dim_size(MPI_Z)      ! map rank number to 2D rank koordinates
    mpi_zrank= mod(mpi_rank,dim_size(MPI_Z))

    ! now compute xoffset
    ! the [x|y]_ranks <> [x|y]offs relation is: ( XXX check this formula!!! XXX )
    !
    ! rank  | 0 1             2             3              4
    ! xoffs | 0 1*XSIZE+BSIZE 2*XSIZE+BSIZE 3*XSIZE+BSIZE  4*XSIZE+BSIZE ...
    ! so:
    if(mpi_xrank/=0) then; mpi_xoffs=mpi_xrank*XSIZE+BSIZE; else; mpi_xoffs=0; endif
    if(mpi_zrank/=0) then; mpi_zoffs=mpi_zrank*ZSIZE+BSIZE; else; mpi_zoffs=0; endif

    ! compute size of global array
    !
    glob_xsize=dim_size(MPI_X)*XSIZE+2*BSIZE
    glob_zsize=dim_size(MPI_Z)*ZSIZE+2*BSIZE

  end subroutine my_cart_init

  !###################################################################

  ! check number of invoked processes
  !
  subroutine my_mpi_check(number_of_procs)
    implicit none
    integer number_of_procs
    ! see how many number of processes are invoked
    !
    call MPI_Comm_size(MPI_COMM_WORLD,mpi_size,mpi_ierr)

    ! check if number of procs is valid (in this example that must be 6)
    !
    if(mpi_size /=number_of_procs) then
      if(mpi_rank==0) then
        write(*,*)'  Please start with ',dim_size(MPI_X),'*',dim_size(MPI_Z),'=',number_of_procs,' processes.'
        write(*,*)'    (you entered ',mpi_size,'processes).'
      endif
      call MPI_Finalize(mpi_ierr)
      stop
    endif
  end subroutine my_mpi_check

  !###################################################################

  ! Map global index to local index.
  ! 
  !
  subroutine my_glob2loc(glob,loc,dim)
    use global ! use mod_params
    implicit none

    integer glob, loc, dim

    integer mpi_irank ! rank pos. for actual dimension (mpi_[x|z]rank)
    integer size      ! size along actual dimension (xsize or zsize)

    ! index no's start at '1'
    if(dim==0) then;      stop;                                endif
    if(dim==1) then;      mpi_irank=mpi_xrank; size=xsize;     endif
    if(dim==2) then;      mpi_irank=mpi_zrank; size=zsize;     endif

    ! there is no index '3' (2D arrays)
    if(dim>=3) then;      stop;                                endif
    
    if(mpi_irank==0) then;      loc = glob
    else;                       loc = glob - BSIZE - size*mpi_irank
    endif
  end subroutine my_glob2loc

  !###################################################################

  ! Function my_index(glob,dim)
  !
  ! Description:
  !   Returns 1, if global index 'glob' resides in the actual
  !   local array, otherwise 0.
  !
  ! Input/Output values:
  !   glob: global index
  !   dim:  dimension, dim=1 : x-dir dim=2: z-dir
  !    you should use MPI_X (=1) and MPI_Z (=2) instead of plain numbers.
  !
  !   returns: 1 or 0 (true or false)
  !
  ! Others:
  !   uses subroutine my_glob2loc()
  !
  function my_index(glob,dim)
    use global ! use mod_params
    implicit none
    integer my_index, glob,dim

    integer loc          ! local index of 'glob' with respect to actual rank
    integer first,last   ! model index range of actual rank
    integer mpi_smaller  ! contains mpi_left or mpi_upper, depending on 'dim'
    integer mpi_bigger   ! contains mpi_right or mpi_lower, depending on 'dim'
    integer size         ! contains xsize or zsize, depending on 'dim'

    ! determine neighbour ranks for actual 'dim'
    !
    if(dim==1) then         ! actual dim is x-axis
      mpi_smaller=mpi_left
      mpi_bigger=mpi_right
      size=xsize
    else                    ! ok, actual dim is z-axis
      mpi_smaller=mpi_upper
      mpi_bigger=mpi_lower
      size=zsize
    endif ! endif dim

    call my_glob2loc(glob,loc,dim)
    ! write(*,*)'xrank=',mpi_xrank,' zrank=',mpi_zrank,' global=',glob,' ->local=',loc

    first=1                           ! first local model index is '1'

    if(mpi_bigger==MPI_PROC_NULL) then  ! no bigger (right/lower) neighbour exists
      last=first+size+2*BSIZE-1         ! second buffer belongs to model
                                        ! (unused mpi buffer belongs to model)
    else                                ! we have a bigger (right/lower) neighbour
      last=first+size+BSIZE-1           ! second buffer needed by mpi, not usable for model.
    endif
    ! write(*,*)'xrank=',mpi_xrank,'zrank=',mpi_zrank,' first=',first,' last=',last

  if(loc>=first .and. loc<=last) then
    my_index=1
  else
    my_index=0
  endif
  end function my_index

  !###################################################################

  function test_function(a)
           integer test_function, a
           test_function = a*3
  end function test_function

  !###################################################################

  ! Function local(glob,dim)
  !
  ! Description:
  !   Returns local index position corresponding
  !   to the given global position.
  !
  ! Input/Output values:
  !   glob: global index
  !   dim:  dimension, dim=1 : x-dir dim=2: z-dir
  !   returns: local index
  !
  ! Others:
  !
  function local(glob,dim)
    use global ! use mod_params
    implicit none

    integer glob,dim,local

    integer mpi_irank ! rank pos. for actual dimension (mpi_[x|z]rank)
    integer size      ! size along actual dimension (xsize or zsize)

    ! index no's start at '1'
    if(dim==0) then; stop;                            endif
    if(dim==1) then; mpi_irank=mpi_xrank; size=xsize; endif
    if(dim==2) then; mpi_irank=mpi_zrank; size=zsize; endif
    ! there is no index '3' in this library (2D arrays)
    if(dim>=3) then; stop;                            endif

    if(mpi_irank==0) then; local = glob
    else;                  local = glob - BSIZE - size*mpi_irank
    endif

  end function local

  !###################################################################

  ! Function glob(loc,dim)
  !
  ! Description:
  !   Returns global index position corresponding
  !   to the given local position.
  !
  ! Input/Output values:
  !   loc: local index
  !   dim:  dimension, dim=1 : x-dir dim=2: z-dir
  !   returns: global index
  !
  ! Others:
  !
  function glob(loc,dim)
    use global ! use mod_params
    implicit none

    integer loc,dim,glob

    integer mpi_irank ! rank pos. for actual dimension (mpi_[x|z]rank)
    integer size      ! size along actual dimension (xsize or zsize)

    if(dim==0) then; stop;    endif                         ! no '0'
    if(dim==1) then; mpi_irank=mpi_xrank; size=xsize; endif
    if(dim==2) then; mpi_irank=mpi_zrank; size=zsize; endif
    if(dim>=3) then; stop;                            endif ! only 2D library

    if(mpi_irank==0) then; glob = loc;
    else;                  glob = loc+BSIZE+size*mpi_irank
    endif
  end function glob 


  !###################################################################

  ! Map global index to local index.
  !
  ! glob: global index
  ! loc:  local index
  ! dim:  dimension, dim=1 : x-dir dim=2: z-dir
  !
  subroutine my_loc2glob(loc,glob,dim)
    use global ! use mod_params
    implicit none
    integer loc
    integer glob
    integer dim

    integer mpi_irank ! rank pos. for actual dimension (mpi_[x|z]rank)
    integer size      ! size along actual dimension (xsize or zsize)

    if(dim==0) then
      stop
    endif                    ! no '0'
    if(dim==1) then
      mpi_irank=mpi_xrank
      size=xsize
    endif
    if(dim==2) then
      mpi_irank=mpi_zrank
      size=zsize
    endif
    if(dim>=3) then
      stop
    endif ! only 2D
    if(mpi_irank==0) then
      glob = loc
    else
      glob = loc+BSIZE+size*mpi_irank
    endif
  end subroutine my_loc2glob

  !###################################################################

  ! do mpi communication
  !
  subroutine mpi_communicate(xlocal)
    use global ! use mod_params
    implicit none
    integer mpi_status(MPI_STATUS_SIZE) ! needed by mpi_sendrecv()
    integer, parameter :: MAX_XZSIZE=ZSIZE  ! for tests only
    !real xlocal(XSIZE+2*BSIZE,ZSIZE+2*BSIZE)
    real xlocal(nx,nz)
    integer i,k            ! array indices
!    real, dimension(MAX_XZSIZE+2*BSIZE,BSIZE) :: sndbuf,rcvbuf  ! buffer for boundary exchange
!   MPI_REAL, dimension((MAX_XZSIZE+2*BSIZE)*BSIZE) :: vsndbuf,vrcvbuf  ! vector buffer
    REAL vsndbuf((MAX_XZSIZE+2*BSIZE)*BSIZE)
    REAL vrcvbuf((MAX_XZSIZE+2*BSIZE)*BSIZE)  ! vector buffer

!    if(it==1) then
!      if(mpi_rank==0) then; open(unit=20,file='mpilog_00.dat',status='unknown'); endif
!      if(mpi_rank==1) then; open(unit=20,file='mpilog_01.dat',status='unknown'); endif
!    endif

!    if(it==20) then; close(20); endif

!!!!    write(*,*)'mpi_communicate() Info: rank=',mpi_rank,' left=',mpi_left,' right=',mpi_right

    !send data of rightmost column to right neighbour
    !the sending is done in 3 steps:
    !  1 filling of send buffer from local matrix
    !  2 invoking of mpi routine for send+receive
    !  3 copying the receive buffer to local matix
    !
    if(mpi_right/=MPI_PROC_NULL) then       ! if right neighbour exists
!!!!      write(*,*)'sending ',ZSIZE+2*BSIZE,' * ',BSIZE,' values'
      do i=1,(ZSIZE+2*BSIZE)            ! loop for boundary-parallel index
        do k=1,BSIZE                    ! loop for index perpendicular to boundary
!          sndbuf(i,k)=xlocal(XSIZE+k,i) ! fill send buffer
          vsndbuf(mapi(i,k) ) =xlocal(XSIZE+k,i) ! fill send buffer
        enddo ! enddo k
      enddo ! enddo i
!    endif ! endif mpi_right ! XXX

!    ! save buffer to file
!    if(mpi_rank==0) then; open(unit=20,file='sndbuf_r_00.dat',status='unknown'); endif
!    if(mpi_rank==1) then; open(unit=20,file='sndbuf_r_01.dat',status='unknown'); endif
!    do i=1,(ZSIZE+2*BSIZE); do k=1,BSIZE; write(20,*)vsndbuf(mapi(i,k)); enddo; enddo
!    close(20)

!    write(20,*)'it=',it,' mpi_communicate() toright mpi_rank ',mpi_rank,' ->  ',mpi_right,' sends ',sndbuf(1,1)
    call MPI_Sendrecv(vsndbuf,   &  ! buffer to send
       BSIZE*(ZSIZE+2*BSIZE),   &  ! number of elements to send
                    MPI_REAL,   &  ! data type to send
                   mpi_right,   &  ! rank of destination process
                           0,   &  ! send tag
                      vrcvbuf,   &  ! buffer for receive
       BSIZE*(ZSIZE+2*BSIZE),   &  ! number of elements to receive
                    MPI_REAL,   &  ! data type to receive
                   mpi_right,   &  ! rank of source process
                           0,   &  ! receive tag
              MPI_COMM_WORLD,   &  ! mpi communicator
                  mpi_status,   &  ! status of operation
                    mpi_ierr)      ! error variable

!   if(mpi_right/=MPI_PROC_NULL) then   ! if right neighbour exists !XXX
!!!!      write(*,*)'receiving ',ZSIZE+2*BSIZE,' * ',BSIZE,' values'
      do i=1,(ZSIZE+2*BSIZE)            ! loop for boundary-parallel index
        do k=1,BSIZE                    ! loop for index perpendicular to boundary
!          xlocal(XSIZE+BSIZE+k,i)=rcvbuf(i,k)  ! read receive buffer
          xlocal(XSIZE+BSIZE+k,i)=vrcvbuf(mapi(i,k)) ! fill send buffer
        enddo ! enddo k
      enddo ! enddo i
!    write(20,*)'it=',it,' mpi_communicate() fromright mpi_rank ',mpi_rank,' gets ',rcvbuf(1,1)

!    ! save buffer to file
!    if(mpi_rank==0) then; open(unit=20,file='rcvbuf_r_00.dat',status='unknown'); endif
!    if(mpi_rank==1) then; open(unit=20,file='rcvbuf_r_01.dat',status='unknown'); endif
!    do i=1,(ZSIZE+2*BSIZE); do k=1,BSIZE; write(20,*)vrcvbuf(mapi(i,k)); enddo; enddo
!    close(20)

    endif ! endif mpi_right


    ! send data of leftmost column to left neighbour
    !
    if(mpi_left/=MPI_PROC_NULL) then
!!!!      write(*,*)'sending ',ZSIZE+2*BSIZE,' * ',BSIZE,' values'
      do i=1,(ZSIZE+2*BSIZE)            ! loop for boundary-parallel index
        do k=1,BSIZE                    ! loop for index perpendicular to boundary
!          sndbuf(i,k)=xlocal(BSIZE+k,i) ! fill send buffer
          vsndbuf(mapi(i,k) )=xlocal(BSIZE+k,i) ! fill send buffer
        enddo ! enddo k
      enddo ! enddo i
!    endif ! endif mpi_left ! XXX

!    ! save buffer to file
!    if(mpi_rank==0) then; open(unit=20,file='sndbuf_l_00.dat',status='unknown'); endif
!    if(mpi_rank==1) then; open(unit=20,file='sndbuf_l_01.dat',status='unknown'); endif
!    do i=1,(ZSIZE+2*BSIZE); do k=1,BSIZE; write(20,*)vsndbuf(mapi(i,k)); enddo; enddo
!    close(20)

!    write(20,*)'it=',it,' mpi_communicate() toleft  mpi_rank ',mpi_rank,' ->  ',mpi_left,' sends ',sndbuf(1,1)
    call MPI_Sendrecv(vsndbuf,BSIZE*(ZSIZE+2*BSIZE),MPI_REAL,mpi_left,0, &
                      vrcvbuf,BSIZE*(ZSIZE+2*BSIZE),MPI_REAL,mpi_left,0, &
                      MPI_COMM_WORLD, mpi_status,mpi_ierr)

!   if(mpi_left/=MPI_PROC_NULL) then ! XXX
!!!!      write(*,*)'receiving ',ZSIZE+2*BSIZE,' * ',BSIZE,' values'
      do i=1,(ZSIZE+2*BSIZE)            ! loop for boundary-parallel index
        do k=1,BSIZE                    ! loop for index perpendicular to boundary
!          xlocal(k,i)=rcvbuf(i,k)
          xlocal(k,i)=vrcvbuf(mapi(i,k) ) ! fill send buffer
        enddo ! enddo k
      enddo ! enddo i
!    write(20,*)'it=',it,' mpi_communicate() fromleft  mpi_rank ',mpi_rank,'gets ',sndbuf(1,1)

    ! save buffer to file
!    if(mpi_rank==0) then; open(unit=20,file='rcvbuf_l_00.dat',status='unknown'); endif
!    if(mpi_rank==1) then; open(unit=20,file='rcvbuf_l_01.dat',status='unknown'); endif
!    do i=1,(ZSIZE+2*BSIZE); do k=1,BSIZE; write(20,*)vrcvbuf(mapi(i,k)); enddo; enddo
!    close(20)

    endif ! endif mpi_left

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!
!!!    ! send/receive data of uppermost rows to/from upper neighbour
!!!    !
!!!    if(mpi_upper/=MPI_PROC_NULL) then
!!!      do i=1,(XSIZE+2*BSIZE)              ! loop for boundary-parallel index
!!!        do k=1,BSIZE                      ! loop for index perpendicular to boundary
!!!          sndbuf(i,k)=xlocal(i,BSIZE+k)
!!!        enddo ! enddo k
!!!      enddo ! enddo i
!!!!    endif ! endif mpi_upper ! XXX
!!!
!!!    call MPI_Sendrecv(sndbuf,BSIZE*(XSIZE+2*BSIZE),MPI_REAL,mpi_upper,0, &
!!!                      rcvbuf,BSIZE*(XSIZE+2*BSIZE),MPI_REAL,mpi_upper,0, &
!!!                      MPI_COMM_WORLD,mpi_status,mpi_ierr)
!!!
!!!!    if(mpi_upper/=MPI_PROC_NULL) then ! XXX
!!!      do i=1,(XSIZE+2*BSIZE)              ! loop for boundary-parallel index
!!!        do k=1,BSIZE                      ! loop for index perpendicular to boundary
!!!          xlocal(i,k)=rcvbuf(i,k)
!!!        enddo ! enddo k
!!!      enddo ! enddo i
!!!    endif ! endif mpi_upper
!!!
!!!    ! send/receive data of lowermost rows to/from lower neighbour
!!!    !
!!!    if(mpi_lower/=MPI_PROC_NULL) then
!!!      do i=1,(XSIZE+2*BSIZE)              ! loop for boundary-parallel index
!!!        do k=1,BSIZE                      ! loop for index perpendicular to boundary
!!!          sndbuf(i,k)=xlocal(i,ZSIZE+k)
!!!        enddo ! enddo k
!!!      enddo ! enddo i
!!!!    endif ! endif mpi_lower ! XXX
!!!
!!!    call MPI_Sendrecv(sndbuf,BSIZE*(XSIZE+2*BSIZE),MPI_REAL,mpi_lower,0, &
!!!                      rcvbuf,BSIZE*(XSIZE+2*BSIZE),MPI_REAL,mpi_lower,0, &
!!!                      MPI_COMM_WORLD,mpi_status,mpi_ierr)
!!!
!!!!    if(mpi_lower/=MPI_PROC_NULL) then ! XXX
!!!      do i=1,(XSIZE+2*BSIZE)              ! loop for boundary-parallel index
!!!        do k=1,BSIZE                      ! loop for index perpendicular to boundary
!!!          xlocal(i,ZSIZE+BSIZE+k)=rcvbuf(i,k)
!!!        enddo ! enddo k
!!!      enddo ! enddo i
!!!    endif ! endif mpi_lower
  end subroutine mpi_communicate

  !###################################################################

  ! Function my_position(pos,dim)
  !
  ! Description:
  !   Returns 1, if physical location 'pos' resides in the actual
  !   local model part, otherwise 0.
  !
  ! Input/Output values:
  !   pos: physical position (e.g. in [m] or [km])
  !   dim:  dimension, dim=1 : x-dir dim=2: y-dir
  !
  !   returns: 1 or 0 (true or false)
  !
  ! Others:
  !   uses subroutine 
  !   and  functions  grid2pos()
  !
  function my_position(pos,dim)
    use global
    implicit none

    integer my_position,dim
    real pos

    ! check this for positions near the border (within the mpi buffer) !!!
    if(dim==1) then
      if( (grid2pos(glob(1,1),dtheta) <= pos) .and. grid2pos(glob(xsize+BSIZE,1),dtheta) >= pos ) then
        my_position=1
      else
        my_position=0
      endif
    else
      write(*,*)'error in function my_position() : currently, only x-dimension supported'
      my_position=1 ! currently, no decomposition along x
    endif
  end function my_position

  !###################################################################

  !###################################################################

  ! Function grid2pos(grd,da)
  !
  ! Description:
  !   Converts grid index to physical position ( [m] or [km] )
  !
  ! Input/Output values:
  !   grd: grid index
  !   da:  grid spacing (dtheta, ...)
  !   returns: physical position (e.g. in [m] or [km]).
  !
  ! Others:
  !
  function grid2pos(grd,da)
    implicit none

    integer grd
    real da,grid2pos

    grid2pos=(grd-1)*da

  end function grid2pos

  function mapi(ii,ik)
    use global, only : ZSIZE,BSIZE
    implicit none
    integer mapi,ii,ik
    mapi = ii+(ik-1)*(ZSIZE+2*BSIZE)
  end function mapi

end module mod_cart2D



