MODULE circle
CONTAINS

  !========================================================================================!
  !
  !  SETCIRCLE
  !
  !  Set parameters for circular anomaly
  !
  !========================================================================================!
  SUBROUTINE setcircle(Rcenter,EPcircle,Rcircle,circle_v,circle_rho)
  IMPLICIT NONE
  REAL, INTENT(OUT) :: Rcenter        !Radius to center of circular anomaly (km)
  REAL, INTENT(OUT) :: EPcircle       !Epicentral Distance to center of anomaly (deg)
  REAL, INTENT(OUT) :: Rcircle        !Radius of circular anomaly (km)
  REAL, INTENT(OUT) :: circle_v       !Vs inside circle (% difference from surrounding mantle)
  REAL, INTENT(OUT) :: circle_rho     !Density inside circle (% difference from mantle)

  !Set parameters for circle - to be used by makecircle
  Rcenter      = 4500.0
  EPcircle     = 32.5
  Rcircle      = 200.0
  circle_v     = -5.0
  circle_rho   = -2.0

  END SUBROUTINE setcircle
  !========================================================================================!


  !========================================================================================!
  !
  !  CIRCLEDRIVER
  !
  !  Driver routine to produce circular anomaly
  !
  !========================================================================================!
  SUBROUTINE circledriver(rho,mu1,mu2)
  USE global, only: nx, nz, r1, theta1, r2, theta2, pi
  IMPLICIT NONE
  REAL, DIMENSION(nx,nz), INTENT(INOUT) :: rho, mu1, mu2
  REAL :: trad, ttheta, Vsin, Rhoin, Vsout, Rhoout
  INTEGER :: z, x

  CALL sh_prem

  DO z=1,nz
    DO x=1,nx

      trad = r1(x,z)/1000.               !trad = radius in km
      ttheta = theta1(x,z)*(180.0/pi)    !ttheta = theta in deg
      Vsin = mu1(x,z)                    !Vs (km/sec)
      Rhoin = rho(x,z)                   !Rho
      CALL makecircle(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu1(x,z) = Vsout
      rho(x,z) = Rhoout

      trad = r2(x,z)/1000.0
      ttheta = theta2(x,z)*(180.0/pi)
      Vsin = mu2(x,z)
      CALL makecircle(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu2(x,z) = Vsout

    ENDDO
  ENDDO
  rho=rho*1000. ! adjust units
  mu1=mu1*1000.
  mu2=mu2*1000.
  mu1=rho*mu1**2 ! convert vs-> mu
  mu2=rho*mu2**2 ! ( vs=rho*mu^2 )

  END SUBROUTINE circledriver
  !========================================================================================!

  !========================================================================================!
  !
  !  MAKECIRCLE
  !
  !  Subroutine that creates the circular anomaly
  ! 
  !     r          = radius of grid point (km) EXPECTS UNITS == KM!
  !     theta      = theta of grid point (deg) EXPECTS UNITS == DEG!
  !     Vsin       = Current Vs of grid point (km/sec)
  !     Vsout      = Output velocity of grid point (km/sec)
  !     Rhoin      = Current density of grid point (g/cm^3)
  !     Rhoout     = Output density of grid point
  !
  !========================================================================================!
  SUBROUTINE makecircle(r,theta,Vsin,Vsout,Rhoin,Rhoout)
  IMPLICIT NONE
  REAL, INTENT(IN) :: r, theta, Vsin, Rhoin
  REAL, INTENT(OUT) :: Vsout, Rhoout
  REAL, PARAMETER :: dtr=.01745329252222
  REAL :: x, y, xc, yc, d
  REAL :: Rcenter, EPcircle, Rcircle, circle_v, circle_rho

  CALL setcircle(Rcenter,EPcircle,Rcircle,circle_v,circle_rho)

  !Convert r,theta position to x,y
  x = r*sin(theta*dtr)
  y = r*cos(theta*dtr)

  !Find center of circle
  xc = Rcenter*sin(EPcircle*dtr)
  yc = Rcenter*cos(EPcircle*dtr)

  !Calculate distance between current position and center of circle
  d = sqrt(((xc-x)**2 + (yc-y)**2))

  IF (d <= Rcircle) THEN
     Vsout = (Vsin)*(circle_v/100.0) + Vsin
     Rhoout = (Rhoin)*(circle_rho/100.0) + Rhoin
  ELSE
     Vsout = Vsin
     Rhoout = Rhoin
  ENDIF


  END SUBROUTINE makecircle
  !========================================================================================!

END MODULE circle


