MODULE modifycrust
CONTAINS

  !SET PARAMETERS
  !============================================================================!
  SUBROUTINE setcrust(crust_add,crust_epd,crust_width,crust_trans)
  IMPLICIT NONE
  REAL, INTENT(OUT) :: crust_add       !Add 'crust_add' km to base of crust
  REAL, INTENT(OUT) :: crust_epd       !Epicentral distance to center of
                                       !  thickened crust (deg)
  REAL, INTENT(OUT) :: crust_width     !width (km) of thickened region of
                                       !  crust
  REAL, INTENT(OUT) :: crust_trans     !width (km) of regions on side of
                                       !  crust for smooth transition     
                                   
  !Set parameters for crust - to be used by makecrust
  crust_add   = 20.0
  crust_epd   = 40.0
  crust_width = 210.0
  crust_trans = 45.0

  END SUBROUTINE setcrust
  !============================================================================!


  !============================================================================!
  SUBROUTINE crustdriver(rho,mu1,mu2)
  USE global, only: nx, nz, r1, theta1, r2, theta2, pi
  IMPLICIT NONE
  REAL, DIMENSION(nx,nz), INTENT(INOUT) :: rho, mu1, mu2
  REAL :: trad, ttheta, Vsin, Rhoin, Vsout, Rhoout
  INTEGER :: z, x

  CALL sh_prem

  DO z=1,nz
    DO x=1,nx

      trad = r1(x,z)/1000.               !trad = radius in km
      ttheta = theta1(x,z)*(180.0/pi)    !ttheta = theta in deg
      Vsin = mu1(x,z)                    !Vs (km/sec)
      Rhoin = rho(x,z)                   !Rho
      CALL makecrust(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu1(x,z) = Vsout
      rho(x,z) = Rhoout

      trad = r2(x,z)/1000.0
      ttheta = theta2(x,z)*(180.0/pi)
      Vsin = mu2(x,z)
      CALL makecrust(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu2(x,z) = Vsout

    ENDDO
  ENDDO
  rho=rho*1000. ! adjust units
  mu1=mu1*1000.
  mu2=mu2*1000.
  mu1=rho*mu1**2 ! convert vs-> mu
  mu2=rho*mu2**2 ! ( vs=rho*mu^2 )

  END SUBROUTINE crustdriver
  !============================================================================!


  !============================================================================!
  SUBROUTINE makecrust(r,theta,Vsin,Vsout,Rhoin,Rhoout)
  !r          = radius of grid point (km) EXPECTS UNITS == KM!
  !theta      = theta of grid point (deg) EXPECTS UNITS == DEG!
  !Vsin       = Current Vs of grid point (m/sec or km/sec)
  !Vsout      = Output velocity of grid point
  !Rhoin      = Current density of grid point
  !Rhoout     = Output density of grid point
  !
  !  This subroutine will add a thickened crustal region.  It only modifies
  !  the PREM crust to thicken the discontinuity at R=6346.6 km.  It does
  !  not affect the mid-crustal discontinuity in PREM at all.
  !
  IMPLICIT NONE
  REAL, INTENT(IN) :: r, theta, Vsin, Rhoin
  REAL :: crust_add,crust_epd,crust_width, crust_trans
  REAL, INTENT(OUT) :: Vsout, Rhoout
  REAL, PARAMETER :: dtr=.01745329252222
  REAL, PARAMETER :: rtd=57.295779505601
  REAL :: ledge, redge, arg, top, bottom
  REAL :: l1, l2, width, rtem

  CALL setcrust(crust_add,crust_epd,crust_width,crust_trans)

  top = 6346.6
  bottom = top-crust_add

  arg = (crust_width/top)*rtd
  arg = arg/2.0
  ledge = crust_epd - arg + (crust_trans/top)*rtd
  redge = crust_epd + arg - (crust_trans/top)*rtd

  IF (r <= top .AND. r >= bottom .AND. theta > ledge .AND. theta < redge) THEN
    Vsout = 3.9
    Rhoout = 2.9 
  ELSEIF (theta <= ledge .AND. theta >= (ledge-(crust_trans/top)*rtd)) THEN
     l1 = ledge-(crust_trans/top)*rtd
     l2 = ledge
     width = (l2-l1)
     rtem = top -(crust_add/2.0) + (crust_add/2.0)*cos((l1-theta)*3.141592654/width)
     IF (r >= rtem .AND. r <= top) THEN
       Vsout = 3.9
       Rhoout = 2.9 
     ELSE
       Vsout = Vsin
       Rhoout = Rhoin
     ENDIF
  ELSEIF (theta >= redge .AND. theta <= (redge+(crust_trans/top)*rtd)) THEN
     l1 = redge
     l2 = redge+(crust_trans/top)*rtd
     width = (l2-l1)
     rtem = top -(crust_add/2.0) + (crust_add/2.0)*cos((l2-theta)*3.141592654/width)
     IF (r >= rtem .AND. r <= top) THEN
      Vsout = 3.9
      Rhoout = 2.9 
     ELSE
       Vsout = Vsin
       Rhoout = Rhoin
     ENDIF
  ELSE
    Vsout = Vsin
    Rhoout = Rhoin
  ENDIF


  END SUBROUTINE makecrust
  !============================================================================!



END MODULE modifycrust


