MODULE ncoutput
CONTAINS

  !=====================================================================================
  ! 
  !  NCOUTPUT
  !
  !  Module containing subroutines for writing output model files in
  !  in the netCDF format.
  !
  !  by, Michael Thorne
  !  Last Updated: 7/24/07
  !
  !=====================================================================================

  !=====================================================================================
  !
  !  WRITE_NC_MODEL
  !
  !  Subroutine to write out a model file in netCDF format 
  !
  !     Inputs:
  !      o_rad: array of radii (either r1, or r2)
  !    o_theta: array of theta (either theta1, or theta2)
  !      o_mu1: shear modulus (mu1)
  !      o_mu2: shear modulus (mu2)
  !      o_rho: density (rho)
  !        o_q: quality factors (Q1 or Q2)
  !     myrank: rank of current process
  !      otype: 1=ncview; 2=GMT
  !
  !     Note, the subroutine will write Vs1, and Vs2 calculated from mu1 and mu2
  !           but only writes both to the coordinates given by o_rad and o_theta
  !     I include adding which mu variable, because sometimes I want to switch
  !           between mu1 and Mu_1 (the unrelaxed modulus used when attenuation
  !           is switched on).
  !
  !     The option 'otype' describes which format of netCDF file to write out.
  !     I prefer 'otype'=1, which makes checking the model files very quick
  !     and easy using the freeware program 'ncview'.  But, I also include
  !     the option to 'otype'=2 for writing out files compatible with the
  !     Generic Mapping Tools (GMT).  Somewhat annoyingly, GMT expects that
  !     the arrays it reads in should flipped top to bottom from what one
  !     would normally expect.
  !
  !=====================================================================================
  SUBROUTINE write_nc_model(o_rad,o_theta,o_mu1,o_mu2,o_rho,o_q,myrank,otype)
  USE netcdf
  USE global, ONLY: nx, nz, seisfile, model_type
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rad, o_theta, o_mu1, o_mu2
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rho, o_q
  REAL, DIMENSION(nx,nz)   :: odata
  REAL, DIMENSION(nx,nz)   :: wodata
  REAL, DIMENSION(nz,nx)   :: wdata
  REAL, DIMENSION(nx) :: xcoords
  REAL, DIMENSION(nz) :: ycoords
  REAL, DIMENSION(2) :: drange
  INTEGER, INTENT(IN) :: myrank
  INTEGER :: x_varid, y_varid
  INTEGER :: x_dimid, y_dimid
  INTEGER :: ncid, varid_1, varid_2, varid_3, varid_4
  INTEGER :: dimids(2)
  INTEGER :: J, K, ccc, otype
  CHARACTER(LEN=100) :: outfile
  CHARACTER(LEN=8) :: rank_str
  CHARACTER(LEN=*), PARAMETER :: WARNING="Vs2 written on r1,theta1 not r2,theta2"

  write (rank_str,"(I3.3)") myrank

  !Set positions 
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  ycoords(1:nz) = o_rad(1,1:nz)
  ycoords = ycoords/1000.0

  xcoords(1:nx) = o_theta(1:nx,1)
  xcoords = xcoords*(180.0/3.141592654)

  !Set name of output file
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  outfile = seisfile(1:LEN_TRIM(seisfile))//'_rank'//TRIM(adjustl(rank_str))// &
            & '_'//'model.nc'

  !Create the netCDF file.  
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_create(outfile, NF90_CLOBBER, ncid) )

  !Define the dimensions. 
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_def_dim(ncid, "theta", nx, x_dimid) )
  CALL check(nf90_def_dim(ncid, "r", nz, y_dimid) )

  !Define coordinate variables
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_def_var(ncid, "theta", NF90_REAL, x_dimid, x_varid))
  CALL check(nf90_def_var(ncid, "r", NF90_REAL, y_dimid, y_varid))

  !Add units for coordinates
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_put_att(ncid, x_varid, "units", "deg"))
  CALL check(nf90_put_att(ncid, y_varid, "units", "km"))

  CALL check(nf90_put_att(ncid, x_varid, "long_name", "Epicentral Distance"))
  CALL check(nf90_put_att(ncid, y_varid, "long_name", "Radius"))

  drange(1) = MINVAL(xcoords); drange(2) = MAXVAL(xcoords)
  CALL check(nf90_put_att(ncid, x_varid, "actual_range", drange))

  drange(1) = MINVAL(ycoords); drange(2) = MAXVAL(ycoords)
  CALL check(nf90_put_att(ncid, y_varid, "actual_range", drange))


  !Define data variable
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  IF (otype == 1) THEN                !NetCDF file best for ncview
    dimids = (/ y_dimid, x_dimid /)
  ELSEIF (otype == 2) THEN            !NetCDF file best for GMT
    dimids = (/ x_dimid, y_dimid /)
  ENDIF
  CALL check( nf90_def_var(ncid, "Vs1", NF90_FLOAT, dimids, varid_1) )
  CALL check( nf90_def_var(ncid, "Vs2", NF90_FLOAT, dimids, varid_2) )
  CALL check( nf90_def_var(ncid, "Rho", NF90_FLOAT, dimids, varid_3) )
  CALL check( nf90_def_var(ncid, "Q",   NF90_FLOAT, dimids, varid_4) )

  !Add Global attributes
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"Conventions","SHmodel"))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"title","SHaxi model file"))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"history","Created by SHaxi"))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"model_type",model_type))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"my_rank",myrank))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"warning",WARNING))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"node_offset",0))

  !Add data attributes
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_put_att(ncid, varid_1, "field","Vs1, scalar"))
  CALL check(nf90_put_att(ncid, varid_2, "field","Vs2, scalar"))
  CALL check(nf90_put_att(ncid, varid_3, "field","Rho, scalar"))
  CALL check(nf90_put_att(ncid, varid_4, "field","Q,   scalar"))

  CALL check(nf90_put_att(ncid, varid_1, "units","km/sec"))
  CALL check(nf90_put_att(ncid, varid_2, "units","km/sec"))
  CALL check(nf90_put_att(ncid, varid_3, "units","gm/cm^3"))

  CALL check(nf90_put_att(ncid, varid_1, "long_name", "Shear wave velocity"))
  CALL check(nf90_put_att(ncid, varid_2, "long_name", "Shear wave velocity"))
  CALL check(nf90_put_att(ncid, varid_3, "long_name", "Density"))
  CALL check(nf90_put_att(ncid, varid_4, "long_name", "Quality Factor"))

  odata = (sqrt(o_mu1/o_rho))/1000.0
  drange(1) = MINVAL(odata)
  drange(2) = MAXVAL(odata)
  CALL check(nf90_put_att(ncid, varid_1, "actual_range", drange))

  odata = (sqrt(o_mu2/o_rho))/1000.0
  drange(1) = MINVAL(odata)
  drange(2) = MAXVAL(odata)
  CALL check(nf90_put_att(ncid, varid_2, "actual_range", drange))

  drange(1) = (MINVAL(o_rho))/1000.0
  drange(2) = (MAXVAL(o_rho))/1000.0
  CALL check(nf90_put_att(ncid, varid_3, "actual_range", drange))

  drange(1) = MINVAL(o_q)
  drange(2) = MAXVAL(o_q)
  CALL check(nf90_put_att(ncid, varid_4, "actual_range", drange))

  !End Definitions
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_enddef(ncid) )

  !Write Data
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_put_var(ncid, x_varid, xcoords))
  CALL check(nf90_put_var(ncid, y_varid, ycoords))


  !Write Vs1
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  odata = (sqrt(o_mu1/o_rho))/1000.0

  IF (otype == 1) THEN
    DO J=1,nx
      DO K=1,nz
        wdata(K,J) = odata(J,K)
      ENDDO
    ENDDO
    CALL check(nf90_put_var(ncid, varid_1, wdata))
  ELSEIF (otype == 2) THEN
    DO J=1,nx
      DO K=1,nz
        wodata(J,K) = odata(J,nz-K+1)
      ENDDO
    ENDDO
    CALL check(nf90_put_var(ncid, varid_1, wodata))
  ENDIF

  !Write Vs2
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  odata = (sqrt(o_mu2/o_rho))/1000.0

  IF (otype == 1) THEN
    DO J=1,nx
      DO K=1,nz
        wdata(K,J) = odata(J,K)
      ENDDO
    ENDDO
    CALL check(nf90_put_var(ncid, varid_2, wdata))
  ELSEIF (otype == 2) THEN
    DO J=1,nx
      DO K=1,nz
        wodata(J,K) = odata(J,nz-K+1)
      ENDDO
    ENDDO
    CALL check( nf90_put_var(ncid, varid_2, wodata))
  ENDIF


  !Write Rho
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  odata = o_rho/1000.0
  IF (otype == 1) THEN
    DO J=1,nx
      DO K=1,nz
        wdata(K,J) = odata(J,K)
      ENDDO
    ENDDO
    CALL check(nf90_put_var(ncid, varid_3, wdata))
  ELSEIF (otype == 2) THEN
    DO J=1,nx
      DO K=1,nz
        wodata(J,K) = odata(J,nz-K+1)
      ENDDO
    ENDDO

    CALL check( nf90_put_var(ncid, varid_3, wodata))
  ENDIF

  !Write Q
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  odata = o_q
  IF (otype == 1) THEN
    DO J=1,nx
      DO K=1,nz
        wdata(K,J) = odata(J,K)
      ENDDO
    ENDDO
    CALL check(nf90_put_var(ncid, varid_4, wdata))
  ELSEIF (otype == 2) THEN
    DO J=1,nx
      DO K=1,nz
        wodata(J,K) = odata(J,nz-K+1)
      ENDDO
    ENDDO

  CALL check( nf90_put_var(ncid, varid_4, wodata))
  ENDIF

  !Close netCDF file
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_close(ncid))

  END SUBROUTINE write_nc_model
  !=====================================================================================


  !=====================================================================================
  !
  !  WRITE_NC_OPENDX
  !
  !  Subroutine to write out a model file in netCDF format readable by openDX
  !
  !     Inputs:
  !      o_rad: array of radii (either r1, or r2)
  !    o_theta: array of theta (either theta1, or theta2)
  !      o_mu1: shear modulus (mu1)
  !      o_mu2: shear modulus (mu2)
  !      o_rho: density (rho)
  !     myrank: rank of current process
  !
  !     Note, the subroutine will write Vs1, and Vs2 calculated from mu1 and mu2
  !           but only writes both to the coordinates given by o_rad and o_theta
  !     I include adding which mu variable, because sometimes I want to switch
  !           between mu1 and Mu_1 (the unrelaxed modulus used when attenuation
  !           is switched on).
  !
  !     This subroutine writes out every coordinate pair in cartesian coordinates
  !           into the netCDF file and hence produces rather large netCDF files.
  !           Probably this isn't necessary.  The other subroutines can likely
  !           also be used in openDX, but you have to map the coordinates onto
  !           a polar grid. 
  !
  !=====================================================================================
  SUBROUTINE write_nc_opendx(o_rad,o_theta,o_mu1,o_mu2,o_rho,myrank)
  USE netcdf
  USE global, ONLY: nx, nz, seisfile, model_type
  IMPLICIT NONE
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rad, o_theta, o_mu1, o_mu2
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rho
  REAL, DIMENSION(nx,nz)   :: odata
  REAL, DIMENSION(nz,nx)   :: wdata
  REAL, DIMENSION(2,nx*nz) :: positions
  REAL :: x, y
  INTEGER, INTENT(IN) :: myrank
  INTEGER :: x_varid, y_varid
  INTEGER :: x_dimid, y_dimid
  INTEGER :: ncid, varid_1, varid_2, varid_3
  INTEGER :: dimids(2)
  INTEGER :: J, K, L
  CHARACTER(LEN=100) :: outfile
  CHARACTER(LEN=8) :: rank_str
  CHARACTER(LEN=*), PARAMETER :: WARNING="Vs2 written on r1,theta1 not r2,theta2"

  write (rank_str,"(I3.3)") myrank


  !Set name of output file
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  outfile = seisfile(1:LEN_TRIM(seisfile))//'_rank'//TRIM(adjustl(rank_str))// &
            & '_'//'model.nc'

  !Take the input positions (radius and theta) and convert to x and y
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  L = 1
  DO J=1,nx
    DO K=1,nz
      x = (o_rad(J,K)/1000.0)*sin(o_theta(J,K))
      y = (o_rad(J,K)/1000.0)*cos(o_theta(J,K))
      positions(1,L) = x
      positions(2,L) = y
      L = L + 1
    ENDDO
  ENDDO

  !Create the netCDF file.  
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_create(outfile, NF90_CLOBBER, ncid) )

  !Define the dimensions.  
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_def_dim(ncid, "pointnums", nx*nz, x_dimid))
  CALL check(nf90_def_dim(ncid, "axes", 2, y_dimid))
  dimids = (/ y_dimid, x_dimid /)

  !Define coordinate variables
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_def_var(ncid, "grid", NF90_FLOAT, dimids, x_varid))
  CALL check( nf90_def_dim(ncid, "x", nx, x_dimid) )
  CALL check( nf90_def_dim(ncid, "y", nz, y_dimid) )
  dimids = (/ y_dimid, x_dimid /)

  !Define data variable
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check( nf90_def_var(ncid, "Vs1", NF90_FLOAT, dimids, varid_1) )
  CALL check( nf90_def_var(ncid, "Vs2", NF90_FLOAT, dimids, varid_2) )
  CALL check( nf90_def_var(ncid, "Rho", NF90_FLOAT, dimids, varid_3) )

  !Add Global attributes
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"title","SHaxi model"))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"history","Created by SHaxi"))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"model_type",model_type))
  CALL check(nf90_put_att(ncid,NF90_GLOBAL,"warning",WARNING))

  !add attributes
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check( nf90_put_att(ncid, x_varid, "units","km"))

  CALL check( nf90_put_att(ncid, varid_1, "field","Vs1, scalar"))
  CALL check( nf90_put_att(ncid, varid_2, "field","Vs2, scalar"))
  CALL check( nf90_put_att(ncid, varid_3, "field","Rho, scalar"))

  CALL check( nf90_put_att(ncid, varid_1, "positions","grid"))
  CALL check( nf90_put_att(ncid, varid_2, "positions","grid"))
  CALL check( nf90_put_att(ncid, varid_3, "positions","grid"))

  CALL check( nf90_put_att(ncid, varid_1, "units","km/sec"))
  CALL check( nf90_put_att(ncid, varid_2, "units","km/sec"))
  CALL check( nf90_put_att(ncid, varid_3, "units","gm/cm^3"))


  !End Definitions
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check( nf90_enddef(ncid) )

  !Write Data
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  CALL check( nf90_put_var(ncid, x_varid, positions))

  !Write Vs1
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  odata = (sqrt(o_mu1/o_rho))/1000.0
  DO J=1,nx
    DO K=1,nz
      wdata(K,J) = odata(J,K)
    ENDDO
  ENDDO

  CALL check( nf90_put_var(ncid, varid_1, wdata))

  !Write Vs2
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  odata = (sqrt(o_mu2/o_rho))/1000.0
  DO J=1,nx
    DO K=1,nz
      wdata(K,J) = odata(J,K)
    ENDDO
  ENDDO

  CALL check( nf90_put_var(ncid, varid_2, wdata))

  !Write Rho
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  odata = o_rho/1000.0
  DO J=1,nx
    DO K=1,nz
      wdata(K,J) = odata(J,K)
    ENDDO
  ENDDO

  CALL check( nf90_put_var(ncid, varid_3, wdata))

  CALL check(nf90_close(ncid))


  END SUBROUTINE write_nc_opendx
  !=====================================================================================



  !=====================================================================================
  !
  !  CHECK
  !
  !  Check NetCDF Error Status
  !  If error occurs output error code and exit
  !
  !=====================================================================================
  SUBROUTINE check(istatus)
  USE netcdf
  IMPLICIT NONE
  INTEGER(KIND=4), INTENT(IN) :: istatus

  IF (istatus /= nf90_noerr) THEN
    write(*,*) TRIM(nf90_strerror(istatus))
    STOP
  ENDIF
  END SUBROUTINE check
  !=====================================================================================


END MODULE ncoutput

