MODULE ncoutput
CONTAINS

  !=====================================================================================
  ! 
  !  NCOUTPUT
  !
  !  Module containing subroutines for writing output model files in
  !  in the netCDF format.  This is the stub module for when netCDF
  !  is disabled.
  !
  !  by, Michael Thorne
  !  Last Updated: 7/24/07
  !
  !=====================================================================================

  !=====================================================================================
  !
  !  WRITE_NC_MODEL
  !
  !  Subroutine to write out a model file in netCDF format 
  !
  !     Inputs:
  !      o_rad: array of radii (either r1, or r2)
  !    o_theta: array of theta (either theta1, or theta2)
  !      o_mu1: shear modulus (mu1)
  !      o_mu2: shear modulus (mu2)
  !      o_rho: density (rho)
  !        o_q: quality factors (Q1 or Q2)
  !     myrank: rank of current process
  !      otype: 1=ncview; 2=GMT
  !
  !     Note, the subroutine will write Vs1, and Vs2 calculated from mu1 and mu2
  !           but only writes both to the coordinates given by o_rad and o_theta
  !     I include adding which mu variable, because sometimes I want to switch
  !           between mu1 and Mu_1 (the unrelaxed modulus used when attenuation
  !           is switched on).
  !
  !     The option 'otype' describes which format of netCDF file to write out.
  !     I prefer 'otype'=1, which makes checking the model files very quick
  !     and easy using the freeware program 'ncview'.  But, I also include
  !     the option to 'otype'=2 for writing out files compatible with the
  !     Generic Mapping Tools (GMT).  Somewhat annoyingly, GMT expects that
  !     the arrays it reads in should flipped top to bottom from what one
  !     would normally expect.
  !
  !=====================================================================================
  SUBROUTINE write_nc_model(o_rad,o_theta,o_mu1,o_mu2,o_rho,o_q,myrank,otype)
  ! USE netcdf
  USE global, ONLY: nx, nz, seisfile, model_type
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rad, o_theta, o_mu1, o_mu2
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rho, o_q
  INTEGER, INTENT(IN) :: myrank
  INTEGER :: otype
  END SUBROUTINE write_nc_model
  !=====================================================================================


  !=====================================================================================
  !
  !  WRITE_NC_OPENDX
  !
  !  Subroutine to write out a model file in netCDF format readable by openDX
  !
  !     Inputs:
  !      o_rad: array of radii (either r1, or r2)
  !    o_theta: array of theta (either theta1, or theta2)
  !      o_mu1: shear modulus (mu1)
  !      o_mu2: shear modulus (mu2)
  !      o_rho: density (rho)
  !     myrank: rank of current process
  !
  !     Note, the subroutine will write Vs1, and Vs2 calculated from mu1 and mu2
  !           but only writes both to the coordinates given by o_rad and o_theta
  !     I include adding which mu variable, because sometimes I want to switch
  !           between mu1 and Mu_1 (the unrelaxed modulus used when attenuation
  !           is switched on).
  !
  !     This subroutine writes out every coordinate pair in cartesian coordinates
  !           into the netCDF file and hence produces rather large netCDF files.
  !           Probably this isn't necessary.  The other subroutines can likely
  !           also be used in openDX, but you have to map the coordinates onto
  !           a polar grid. 
  !
  !=====================================================================================
  SUBROUTINE write_nc_opendx(o_rad,o_theta,o_mu1,o_mu2,o_rho,myrank)
  ! USE netcdf
  USE global, ONLY: nx, nz, seisfile, model_type
  IMPLICIT NONE
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rad, o_theta, o_mu1, o_mu2
  REAL, DIMENSION(nx,nz), INTENT(IN) :: o_rho
  INTEGER, INTENT(IN) :: myrank


  END SUBROUTINE write_nc_opendx
  !=====================================================================================



  !=====================================================================================
  !
  !  CHECK
  !
  !  Check NetCDF Error Status
  !  If error occurs output error code and exit
  !
  !=====================================================================================
  SUBROUTINE check(istatus)



  END SUBROUTINE check
  !=====================================================================================


END MODULE ncoutput

