MODULE sacoutput
CONTAINS

  SUBROUTINE wasac(outfile,yarray,npts,b,delta,evdp,stdp,gcarc,o,kcmpnm,kstnm,kevnm)
   !
   !Write SAC alphanumeric format data file
   !read into sac with: 'read alpha outfile'
   !
   ! description of input terms:
   ! ===========================
   !
   ! variable      type                description
   !---------    --------------    ---------------------------------------------
   ! outfile   - char,len=90     - name of output SAC file 
   ! yarray    - real,dim(npts)  - array of amplitude values
   ! npts      - int             - number of elements in 'yarray'
   ! b         - real            - begin time of trace (sec)
   ! delta     - real            - time spacing (sec)
   ! evdp      - real            - event depth (km)
   ! stdp      - real            - receiver depth (km)
   ! gcarc     - real            - event-station great circle arc distance (deg)
   ! o         - real            - event origin time (sec relative to 'b')
   ! kcmpnm    - char,len=8      - Component Name (accepts R,T,Z)
   ! kstnm     - char,len=8      - Station Name
   ! kevnm     - char,len=16     - Event Name (i use for labelling model)
   !
   ! date that synthetic was created is stored in header variable
   ! 'KZDATE' as calculated from nzyear and nzjday (using the attached
   !  function julday) 
   !
   ! time of synthetic creation is stored in nzhour, nzmin, nzsec, and nzmsec and
   ! should be displayed in the header variable KZTIME
   !
   ! kcmpnm can be Z, R, or T for Vertical, Radial, or Transverse
   ! cmpaz and cmpinc are determined from kcmpnm
   !
   ! kevnm is a 16 character string i use to write some information about
   ! the current synthetic run. i use kevnm because all other strings are
   ! only 8 characters in length. (typically i write the name of the 
   ! parameter file used in here)
   !
   ! for some reason if i try to write kcmpnm with R,T, or Z, sac turns
   ! off automatic azimuth, distance, baz, etc., calculations.  Hence,
   ! I don't try to write over kcmpnm which in turn gets labelled 'Q'.
   !
   IMPLICIT NONE
   REAL :: rundef
   INTEGER, INTENT(IN) :: npts
   REAL, DIMENSION(npts), INTENT(IN) :: yarray
   REAL, INTENT(IN) :: b, delta, evdp, gcarc, o, stdp
   REAL :: e, depmin, depmax, depmen, evla, evlo, az, baz
   REAL :: stlo, stla, stel, cmpaz, cmpinc
   INTEGER :: iundef
   INTEGER :: nvhdr, iftype
   INTEGER :: J, mm, id, iyyy, nzjday, nzyear, nzhour, nzmin
   INTEGER :: nzsec, nzmsec, jdaysub
   INTEGER :: ltrue, lfalse
   CHARACTER(LEN=90), INTENT(IN) :: outfile
   CHARACTER(LEN=16), INTENT(IN) :: kevnm
   CHARACTER(LEN=8) :: kundef, date
   CHARACTER(LEN=10) :: time
   CHARACTER(LEN=8), INTENT(IN) :: kcmpnm, kstnm

   !default values
   rundef = -12345.0  !undefined real variable
   iundef = -12345    !undefined integer variable
   ltrue = 1          !logical true
   lfalse = 0         !logical false
   kundef = '-12345'  !undefined character variable

   !basic variables that should be filled in:
   nvhdr = 6                 !header version number
   e = b + npts*delta        !end time
   evla = 0.0                !event latitude
   evlo = 0.0                !event longitude
   iftype = 1                !time series file
   stla = 0.0                !station latitude
   stlo = gcarc              !station longitude
   stel = 0.0                !station elevation

   !grab info for kzdate, and kztime
   CALL date_and_time(date,time)  
   read(date(1:4),*) iyyy
   read(date(5:6),*) mm 
   read(date(7:8),*) id
   read(time(1:2),*) nzhour
   read(time(3:4),*) nzmin
   read(time(5:6),*) nzsec
   nzmsec = 0

   nzjday=julday(mm,id,iyyy)
   mm = 1; id = 0
   jdaysub=julday(mm,id,iyyy)
   nzyear=iyyy; nzjday = nzjday - jdaysub

   !label component inclination and azimuth
   cmpinc = 90.0
   cmpaz = 90.0
    
   OPEN(UNIT=777,FILE=outfile,status='replace')
    
      !write real header variables
      !----------------------------------------------------
      !              DELTA  DEPMIN  DEPMAX  SCALE   ODELTA
      write(777,201) delta, rundef, rundef, rundef, rundef

      !              B  E  O  A       INTERNAL
      write(777,201) b, e, o, rundef, rundef 

      !              T0      T1      T2      T3      T4
      write(777,201) rundef, rundef, rundef, rundef, rundef 

      !              T5      T6      T7      T8      T9
      write(777,201) rundef, rundef, rundef, rundef, rundef 

      !              F       RESP0   RESP1   RESP2   RESP3
      write(777,201) rundef, rundef, rundef, rundef, rundef

      !              RESP4   RESP5   RESP6   RESP7   RESP8 
      write(777,201) rundef, rundef, rundef, rundef, rundef 

      !              RESP9   STLA  STLO  STEL  STDP
      write(777,201) rundef, stla, stlo, stel, stdp

      !              EVLA  EVLO  EVEL    EVDP  MAG
      write(777,201) evla, evlo, rundef, evdp, rundef

      !              USER0   USER1   USER2   USER3   USER4
      write(777,201) rundef, rundef, rundef, rundef, rundef

      !              USER5   USER6   USER7   USER8   USER9
      write(777,201) rundef, rundef, rundef, rundef, rundef

      !              DIST    AZ      BAZ     GCARC   INTERNAL
      write(777,201) rundef, rundef, rundef, rundef, rundef

      !              INTERNAL DEPMEN  CMPAZ  CMPINC  XMINIMUM
      write(777,201) rundef,  rundef, cmpaz, cmpinc, rundef

      !              XMAXIMUM  YMINIMUM  YMAXIMUM  UNUSED  UNUSED
      write(777,201) rundef,   rundef,   rundef,   rundef, rundef

      !              UNUSED  UNUSED  UNUSED  UNUSED  UNUSED
      write(777,201) rundef, rundef, rundef, rundef, rundef

        
      !write integer header variables
      !----------------------------------------------------
      !              NZYEAR  NZJDAY  NZHOUR  NZMIN  NZSEC
      write(777,202) nzyear, nzjday, nzhour, nzmin, nzsec 

      !              NZMSEC  NVHDR  NORID   NEVID   NPTS
      write(777,202) nzmsec, nvhdr, iundef, iundef, npts

      !              INTERNAL  NWFID   NXSIZE  NYSIZE  UNUSED
      write(777,202) iundef,   iundef, iundef, iundef, iundef

      !              IFTYPE  IDEP    IZTYPE  UNUSED  IINST
      write(777,202) iftype, iundef, iundef, iundef, iundef

      !              ISTREG  IEVREG  IEVTYP  IQUAL   ISYNTH
      write(777,202) iundef, iundef, iundef, iundef, iundef

      !              IMAGTYP IMAGSRC UNUSED  UNUSED  UNUSED
      write(777,202) iundef, iundef, iundef, iundef, iundef

      !              UNUSED  UNUSED  UNUSED  UNUSED  UNUSED
      write(777,202) iundef, iundef, iundef, iundef, iundef

      !write logical header variables
      !----------------------------------------------------
      !              LEVEN  LPSPOL  LOVROK LCALDA UNUSED
      write(777,202) ltrue, lfalse, ltrue, ltrue, lfalse

      !write character header variables
      !----------------------------------------------------
      !              KSTNM  KEVNM
      write(777,203) kstnm, kevnm

      !              KHOLE   KO      KA
      write(777,204) kundef, kundef, kundef

      !              KT0     KT1     KT2 
      write(777,204) kundef, kundef, kundef

      !              KT3     KT4     KT5
      write(777,204) kundef, kundef, kundef

      !              KT6     KT7     KT8
      write(777,204) kundef, kundef, kundef

      !              KT9     KF      KUSER0
      write(777,204) kundef, kundef, kundef

      !              KUSER1  KUSER2  KCMPNM
      write(777,204) kundef, kundef, kundef

      !              KNETWK  KDATRD  KINST
      write(777,204) kundef, kundef, kundef


      !write data
      !----------------------------------------------------
      write(777,205) yarray

    CLOSE(777)

    201 FORMAT(G15.7,G15.7,G15.7,G15.7,G15.7)
    202 FORMAT(I10,I10,I10,I10,I10)
    203 FORMAT(A8,A16)
    204 FORMAT(A8,A8,A8)
    205 FORMAT(G15.7,G15.7,G15.7,G15.7,G15.7)

  END SUBROUTINE wasac

  FUNCTION julday(mm,id,iyyy)
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: mm, id, iyyy
  INTEGER :: julday
  !gregorian calendar adopted Oct. 15, 1582
  INTEGER, PARAMETER :: IGREG=15+31*(10+12*1582)
  INTEGER :: ja, jm, jy
  jy=iyyy
  !IF (jy == 0) --> error! there is no year zero.
  IF (jy < 0) jy=jy+1
  IF (mm > 2) THEN
    jm=mm+1
  ELSE 
    jy=jy-1
    jm=mm+13
  ENDIF 
  
  julday=floor(365.25*jy)+floor(30.6001*jm)+id+1720995

  !Test whether to change to Gregorian Calendar
  IF (id+31*(mm+12*iyyy) >= IGREG) THEN
    ja=floor(0.01*jy)
    julday=julday+2-ja+floor(0.25*ja)
  ENDIF

  END FUNCTION julday

END MODULE sacoutput
