MODULE simplerift
CONTAINS

  !============================================================================!
  !
  !  SETRIFT
  !
  !============================================================================!
  SUBROUTINE setrift(rift_width,rift_depth,rift_height,rift_bottom,rift_epd,rift_v,rift_rho)
  IMPLICIT NONE
  REAL, INTENT(OUT) :: rift_width  !Total width of rift (km)
  REAL, INTENT(OUT) :: rift_depth  !Depth at which cos curve of rift starts (km)
  REAL, INTENT(OUT) :: rift_height !total amplitude of cos curve (km)
  REAL, INTENT(OUT) :: rift_bottom !very bottom of rift (>= rift_depth) (km)
  REAL, INTENT(OUT) :: rift_epd    !Epicentral Distance where rift starts (deg)
  REAL, INTENT(OUT) :: rift_v      !Rift Velocity (% difference from PREM)
  REAL, INTENT(OUT) :: rift_rho    !Rift Density (% difference from PREM

  rift_width  = 400.0
  rift_depth  = 150.0
  rift_height = 100.0
  rift_bottom = 200.0
  rift_epd    = 40.0
  rift_v      = -10.0
  rift_rho    = 0.0

  IF (rift_bottom < rift_depth) THEN
    write(420,*) "Error:  rift_bottom must be >= rift_depth"
  ENDIF

  END SUBROUTINE setrift
  !============================================================================!


  !============================================================================!
  !
  !  RIFTDRIVER
  !
  !============================================================================!
  SUBROUTINE riftdriver(rho,mu1,mu2)
  USE global, only: nx, nz, r1, theta1, r2, theta2, pi
  IMPLICIT NONE
  REAL, DIMENSION(nx,nz), INTENT(INOUT) :: rho, mu1, mu2
  REAL :: trad, ttheta, Vsin, Rhoin, Vsout, Rhoout
  INTEGER :: z, x

  CALL sh_prem

  DO z=1,nz
    DO x=1,nx

      trad = r1(x,z)/1000.               !trad = radius in km
      ttheta = theta1(x,z)*(180.0/pi)    !ttheta = theta in deg
      Vsin = mu1(x,z)                    !Vs (km/sec)
      Rhoin = rho(x,z)                   !Rho
      CALL makerift(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu1(x,z) = Vsout
      rho(x,z) = Rhoout

      trad = r2(x,z)/1000.0
      ttheta = theta2(x,z)*(180.0/pi)
      Vsin = mu2(x,z)
      CALL makerift(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu2(x,z) = Vsout

    ENDDO
  ENDDO
  rho=rho*1000. ! adjust units
  mu1=mu1*1000.
  mu2=mu2*1000.
  mu1=rho*mu1**2 ! convert vs-> mu
  mu2=rho*mu2**2 ! ( vs=rho*mu^2 )

  END SUBROUTINE riftdriver
  !============================================================================!


  !============================================================================!
  !
  !  MAKERIFT
  !
  !============================================================================!
  SUBROUTINE makerift(r,theta,Vsin,Vsout,Rhoin,Rhoout)
  !r          = radius of grid point (km) EXPECTS UNITS == KM!
  !theta      = theta of grid point (deg) EXPECTS UNITS == DEG!
  !Vsin       = Current Vs of grid point (m/sec or km/sec)
  !Vsout      = Output velocity of grid point
  !Rhoin      = Current density of grid point
  !Rhoout     = Output density of grid point
  IMPLICIT NONE
  REAL, INTENT(IN) :: r, theta, Vsin, Rhoin
  REAL, INTENT(OUT) :: Vsout, Rhoout
  REAL :: rift_width, rift_depth, rift_height, rift_bottom
  REAL :: rift_epd, rift_v, rift_rho
  REAL, PARAMETER :: dtr=.01745329252222
  REAL, PARAMETER :: rtd=57.295779505601
  REAL :: edge_t, edge_r1, edge_r2, tt1, arg
  REAL :: r1, r2, v1, v2, m, V
  REAL :: x1, x2, d1, angle

  CALL setrift(rift_width,rift_depth,rift_height,rift_bottom,rift_epd,rift_v,rift_rho)

  edge_t = rift_epd + (rift_width/(6371.0-rift_depth))*(rtd)
  edge_r1 = (6371.0 - rift_bottom)

  tt1 = theta - rift_epd
  arg = tt1*(3.141592654/ ((rift_width/(6371.0-rift_depth))*(rtd)))
  edge_r2 = (6371.0-rift_depth) + sin(arg)*rift_height

  IF (theta >= rift_epd .AND. theta <= edge_t .AND. r >= edge_r1 .AND. r <= edge_r2) THEN

    IF (r >= (6371.0-rift_bottom) .AND. r <= (6371.0-rift_depth)) THEN 
      !bottom region
      x1 = (theta-rift_epd)*dtr*(6371.0-rift_depth)
      x1 = ABS((rift_width/2.0) - x1)
      x2 = (rift_width/2.0)-x1
      d1 = x2/(x1+x2)

      v1 = 0.0
      v2 = rift_v
      r1 = 6371.0-rift_bottom
      r2 = 6371.0-rift_depth
      m = (v2-v1)/(r2-r1)
      V = m*(r-r1)+v1

      Vsout = (Vsin)*(d1)*(V/100.0) + Vsin

      v1 = 0.0
      v2 = rift_rho
      r1 = 6371.0-rift_bottom
      r2 = 6371.0-rift_depth
      m = (v2-v1)/(r2-r1)
      V = m*(r-r1)+v1

      Rhoout = (Rhoin)*(d1)*(V/100.0) + Rhoin

    ELSE                              !
      !Actual rift region
      x1 = (theta-rift_epd)*dtr*(6371.0-rift_depth)
      x1 = ABS((rift_width/2.0) - x1)

      angle = asin( (r-(6371.0-rift_depth))/rift_height)
      x2 = angle*(rift_width)/3.141592654
      x2 = (rift_width/2.0)-(x1+x2)

      d1 = x2/(x1+x2)

      Vsout = (Vsin)*d1*(rift_v/100.0) + Vsin
      Rhoout = (Rhoin)*d1*(rift_rho/100.0) + Rhoin
    ENDIF

  ELSE
    Vsout = Vsin
    Rhoout = Rhoin
  ENDIF
 
  END SUBROUTINE makerift
  !============================================================================!

END MODULE simplerift


