MODULE simpleslab
CONTAINS

  !SET PARAMETERS
  !============================================================================!
  SUBROUTINE setslab(slab_dip,slab_dist,slab_len,slab_thick,slab_v,   &
                   & slab_rho,gradient)
  IMPLICIT NONE
  REAL, INTENT(OUT) :: slab_dip        !Dip of slab from horizontal (deg)
  REAL, INTENT(OUT) :: slab_dist       !Epicentral Distance to slab (deg)
  REAL, INTENT(OUT) :: slab_len        !Length of slab (end to end) (km)
  REAL, INTENT(OUT) :: slab_thick      !Thickness of slab (km)
  REAL, INTENT(OUT) :: slab_v          !Slab velociy (% difference from mantle)
  REAL, INTENT(OUT) :: slab_rho        !Slab density (% difference from mantle)
  INTEGER, INTENT(OUT) :: gradient     !0=no gradient; 1=gradient


  !Set parameters for slab - to be used by makeslab
  slab_dip   = 50.0
  slab_dist  = -2.75
  slab_len   = 2500.0
  slab_thick = 200.0
  slab_v     = 4.0
  slab_rho   = 0.0
  gradient   = 1

  END SUBROUTINE setslab
  !============================================================================!


  !============================================================================!
  SUBROUTINE slabdriver(rho,mu1,mu2)
  USE global, only: nx, nz, r1, theta1, r2, theta2, pi
  IMPLICIT NONE
  REAL, DIMENSION(nx,nz), INTENT(INOUT) :: rho, mu1, mu2
  REAL :: trad, ttheta, Vsin, Rhoin, Vsout, Rhoout
  INTEGER :: z, x

  CALL sh_prem

  DO z=1,nz
    DO x=1,nx

      trad = r1(x,z)/1000.               !trad = radius in km
      ttheta = theta1(x,z)*(180.0/pi)    !ttheta = theta in deg
      Vsin = mu1(x,z)                    !Vs (km/sec)
      Rhoin = rho(x,z)                   !Rho
      CALL makeslab(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu1(x,z) = Vsout
      rho(x,z) = Rhoout

      trad = r2(x,z)/1000.0
      ttheta = theta2(x,z)*(180.0/pi)
      Vsin = mu2(x,z)
      CALL makeslab(trad,ttheta,Vsin,Vsout,Rhoin,Rhoout)
      mu2(x,z) = Vsout

    ENDDO
  ENDDO
  rho=rho*1000. ! adjust units
  mu1=mu1*1000.
  mu2=mu2*1000.
  mu1=rho*mu1**2 ! convert vs-> mu
  mu2=rho*mu2**2 ! ( vs=rho*mu^2 )

  END SUBROUTINE slabdriver
  !============================================================================!


  !============================================================================!
  SUBROUTINE makeslab(r,theta,Vsin,Vsout,Rhoin,Rhoout)
  !r          = radius of grid point (km) EXPECTS UNITS == KM!
  !theta      = theta of grid point (deg) EXPECTS UNITS == DEG!
  !Vsin       = Current Vs of grid point (m/sec or km/sec)
  !Vsout      = Output velocity of grid point
  !Rhoin      = Current density of grid point
  !Rhoout     = Output density of grid point
  IMPLICIT NONE
  REAL, DIMENSION(2,2) :: slab, slabt, rmat
  REAL, INTENT(IN) :: r, theta, Vsin, Rhoin
  REAL :: slab_v, slab_len, slab_thick, slab_dip, slab_dist, slab_rho
  REAL, INTENT(OUT) :: Vsout, Rhoout
  REAL, PARAMETER :: dtr=.01745329252222
  REAL :: del, x1, y1, x2, y2, alpha, xoff, yoff
  REAL :: slab_distr
  REAL :: m, xG, xL, yG, x, y, w
  INTEGER :: gradient

  CALL setslab(slab_dip,slab_dist,slab_len,slab_thick,slab_v,slab_rho,gradient)

  !Convert r,theta position to x,y
  x = r*sin(theta*dtr)
  y = r*cos(theta*dtr)

  !Define bottom of slab w.r.t. origin of axis
  del = 90.0 + slab_dip
  del = del*dtr

  x1 = 0.0
  y1 = 0.0

  x2 = slab_len*sin(del)
  y2 = slab_len*cos(del)

  y1 = y1+6371.0
  y2 = y2+6371.0

  slab(1,1) = x1
  slab(2,1) = y1
  slab(1,2) = x2
  slab(2,2) = y2

  !Define top of slab
  alpha = -del
  xoff = slab_thick*cos(alpha)
  yoff = slab_thick*sin(alpha)

  slabt(1,1) = x1-xoff
  slabt(2,1) = y1-yoff
  slabt(1,2) = x2-xoff
  slabt(2,2) = y2-yoff

  !Rotate coordinates to desired epicentral distance of slab
  slab_distr = slab_dist*dtr
  rmat(1,1) = cos(slab_distr)
  rmat(2,1) = -sin(slab_distr)
  rmat(1,2) = sin(slab_distr)
  rmat(2,2) = cos(slab_distr)

  slab = MATMUL(rmat,slab)
  slabt = MATMUL(rmat,slabt)

  !Define Lines bounding slab
  !line 1
  x1 = slab(1,1)
  x2 = slab(1,2)
  y1 = slab(2,1)
  y2 = slab(2,2)
  m  = (y2-y1)/(x2-x1)
  xG = (y-y1)/m + x1

  !line 2
  x1 = slabt(1,1)
  x2 = slabt(1,2)
  y1 = slabt(2,1)
  y2 = slabt(2,2)
  m  = (y2-y1)/(x2-x1)
  xL = (y-y1)/m + x1

  !line 3
  x1 = slab(1,2)
  y1 = slab(2,2)
  x2 = slabt(1,2)
  y2 = slabt(2,2)
  m  = (y2-y1)/(x2-x1)
  yG = m*(x-x1)+y1

  IF (x >= xG .AND. x <= xL .AND. r <= 6371.0 .AND. y > yG) THEN
     
     IF (gradient == 1) THEN   !Call gradient
       CALL slabgrad(slab,x,y,slab_thick,w)
     ELSE
       w = 1.0
     ENDIF

     Vsout = (Vsin)*(w)*(slab_v/100.0) + Vsin
     Rhoout = (Rhoin)*(w)*(slab_rho/100.0) + Rhoin
  ELSE
     Vsout = Vsin
     Rhoout = Rhoin
  ENDIF


  END SUBROUTINE makeslab
  !============================================================================!


  !============================================================================!
  SUBROUTINE slabgrad(slab,x3,y3,slab_thick,weight)
  IMPLICIT NONE
  REAL, DIMENSION(2,2), INTENT(IN) :: slab
  REAL, INTENT(IN) :: x3, y3, slab_thick
  REAL, INTENT(OUT) :: weight
  REAL :: x1, y1, x2, y2, u, temp
  REAL :: sigma, arg

  !First find the distance between the line defining the bottom of the slab
  ! and the point (x3,y3)
  x1 = slab(1,1)
  x2 = slab(1,2)
  y1 = slab(2,1)
  y2 = slab(2,2)

  temp = sqrt((x2-x1)**2 + (y2-y1)**2)
  u = ((x2-x1)*(y1-y3) - (x1-x3)*(y2-y1))/temp
  u = abs(u)
 
  !determine gaussian weighting
  sigma = slab_thick/(2.0*3.141592654)
  arg = 1.0/sqrt((2.0*3.141592654*sigma*sigma))
  weight = arg*exp(-((u-slab_thick/2.0)**2)/(2.0*sigma**2))
  weight = weight/arg

  END SUBROUTINE slabgrad
  !============================================================================!



END MODULE simpleslab


