MODULE tomogrand
CONTAINS

  !SET PARAMETERS
  !============================================================================!
  SUBROUTINE setgrand(xfile)
  IMPLICIT NONE
  INTEGER :: ierr
  CHARACTER(LEN=100), INTENT(OUT) :: xfile

  !Set the name of the file containing the base cross-section.
  !  This should be the output file from the program Grand_Xsctn.f90
  !
  xfile = 'Base_xsctn_grand2002'

  !Check to see that file actually exists before we go on.
  write(420,*) " Checking if file '", TRIM(ADJUSTL(xfile)), "' exists..."
  OPEN(UNIT=888,FILE=xfile,STATUS='OLD',IOSTAT=ierr)

  IF (ierr /= 0) THEN   
    write(420,*) " Error:  File '", TRIM(ADJUSTL(xfile)), "' does not exist..."
  ELSE
    write(420,*) " It does!"
  ENDIF

  CLOSE(888)

  END SUBROUTINE setgrand
  !============================================================================!


  !============================================================================!
  SUBROUTINE tomodriver(rho,mu1,mu2)
  USE global, only: nx, nz, r1, theta1, r2, theta2, pi
  IMPLICIT NONE
  REAL, DIMENSION(nx,nz), INTENT(INOUT) :: rho, mu1, mu2
  REAL, DIMENSION(3,3982) :: model       !read x-section into this array
  REAL :: trad, ttheta, Vsin, Rhoin, Vsout, Rhoout
  REAL :: stlat, stlon, evlat, evlon
  INTEGER :: z, x, nxpts
  INTEGER, PARAMETER :: nrsteve=3982     !# of rec's in x-section file
  CHARACTER(LEN=100) :: xfile

  CALL sh_prem
  CALL setgrand(xfile)
  CALL readx(xfile,nrsteve,model)

  DO z=1,nz
    DO x=1,nx

      trad = r1(x,z)/1000.               !trad = radius in km
      ttheta = theta1(x,z)*(180.0/pi)    !ttheta = theta in deg
      CALL hardvel(trad,ttheta,model,nrsteve,Vsout)
      mu1(x,z) = Vsout

      trad = r2(x,z)/1000.0
      ttheta = theta2(x,z)*(180.0/pi)
      CALL hardvel(trad,ttheta,model,nrsteve,Vsout)
      mu2(x,z) = Vsout

    ENDDO
  ENDDO
  rho=rho*1000. ! adjust units
  mu1=mu1*1000.
  mu2=mu2*1000.
  mu1=rho*mu1**2 ! convert vs-> mu
  mu2=rho*mu2**2 ! ( vs=rho*mu^2 )

  END SUBROUTINE tomodriver
  !============================================================================!

  !============================================================================!
  !
  !  READX
  !
  !   read in cross-section file
  !   xfile is name of cross-section file to read in
  !   must be of format (radius, theta, velocity [%])
  !   nr = number of lines in file 'xfile'
  !   model = output array in exact same format as input
  !   file 'xfile'
  !
  !============================================================================!
  SUBROUTINE readx(xfile,nr,model)
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: nr
  REAL, DIMENSION(3,nr), INTENT(OUT) :: model
  INTEGER :: J
  CHARACTER(len=100), INTENT(IN) :: xfile

  OPEN(UNIT=888,FILE=xfile)
  DO J=1,nr
    READ(888,*) model(1,J), model(2,J), model(3,J)
  ENDDO
  CLOSE(888)

  END SUBROUTINE readx
  !============================================================================!

  !============================================================================!
  !
  !  HARDVEL
  !
  !   determine actual velocity value for 'model' read in
  !   with readx, at radius='r' and angle='theta'
  !   Output 'Vs' is the actual velocity (km/sec) for
  !   Steve's model at the given point.  This subroutine
  !   linearly interpolates between the given velocities
  !   at neighboring theta values.  (linear interpolation
  !   in depth is performed in the function refvel)
  !
  !============================================================================!
  SUBROUTINE hardvel(r,theta,model,nr,Vs)
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: nr
  REAL, DIMENSION(3,nr), INTENT(IN) :: model
  REAL, DIMENSION(23) :: bds
  REAL, INTENT(OUT) :: Vs
  REAL, INTENT(IN) :: r, theta
  REAL :: vref, mintheta, maxtheta, m
  REAL :: mindif, nearr, neart1, neart2
  REAL :: neartg, neartl, mindif2, Vs1, Vs2
  INTEGER :: J

  !find reference velocity for input radius
  vref = refvel(r)

  !find associated 'model' radius
  mindif = 9999999.9
  DO J=1,nr
    IF (ABS(model(2,J) - r) < mindif) THEN
      mindif = ABS(model(2,J) - r)
      nearr = model(2,J)
    ENDIF
  ENDDO
  
  !find associated 'model' theta
  mindif = 9999999.9
  DO J=1,nr
    IF (ABS(model(1,J) - theta) < mindif) THEN
      mindif = ABS(model(1,J) - theta)
      neart1 = model(1,J)
    ENDIF
  ENDDO

  !find bounding 'model' theta
  mindif = 9999999.9
  mindif2 = 9999999.9
  DO J=1,nr
    IF (ABS(model(1,J) - neart1) < mindif .and. model(1,J) < neart1) THEN
      mindif = ABS(model(1,J) - neart1)
      neartl = model(1,J)
    ENDIF
    IF (ABS(model(1,J) - neart1) < mindif2 .and. model(1,J) > neart1) THEN
      mindif2 = ABS(model(1,J) - neart1)
      neartg = model(1,J)
    ENDIF
  ENDDO

  IF (theta <= neart1) THEN
    neart2 = neartl
  ELSEIF (theta > neart1) THEN
    neart2 = neartg
  ENDIF

  mintheta = MINVAL(model(1,:))
  maxtheta = MAXVAL(model(1,:))

  !Determine Vs for theta, r
  IF (theta > maxtheta) THEN
    !use value for neart1
    DO J=1,nr
      IF (model(1,J) == neart1 .and. model(2,J) == nearr) THEN
        Vs = vref + vref*(model(3,J)/100.0)
      ENDIF
    ENDDO
  ELSEIF (theta <= mintheta) THEN
    !use value for neart1
    DO J=1,nr
      IF (model(1,J) == neart1 .and. model(2,J) == nearr) THEN
        Vs = vref + vref*(model(3,J)/100.0)
      ENDIF
    ENDDO
  ELSE
    !interpolate between neart1 and neart2
    DO J=1,nr
      IF (model(1,J) == neart1 .and. model(2,J) == nearr) THEN
         Vs1 = vref + vref*(model(3,J)/100.0)
      ELSEIF (model(1,J) == neart2 .and. model(2,J) == nearr) THEN
         Vs2 = vref + vref*(model(3,J)/100.0)
      ENDIF
    ENDDO
    m = (neart2 - neart1)/(Vs2 - Vs1)
    IF (m == 0.0) THEN
      write(*,*) "ERROR: divide by zero"
    ENDIF
    Vs = Vs1 + (theta - neart1)/m
  ENDIF

  END SUBROUTINE hardvel
  !============================================================================!


  !============================================================================!
  !
  !  REFVEL
  !
  !    function returns the 1-d reference velocity
  !    for the given radius for steve's 2002
  !    global tomography model.
  !    [usage:  velocity=refvel(radius)]
  !
  !============================================================================!
  FUNCTION refvel(radius)
  IMPLICIT NONE
  REAL, DIMENSION(1:298) :: radii, svel, dif
  REAL, INTENT(IN) :: radius
  REAL :: m, b
  REAL :: refvel
  INTEGER, DIMENSION(1) :: J
  INTEGER :: K, L

  radii(1:10)=(/6371.0,6367.5,6367.0,6337.0,6335.0,6311.0,6296.0,6271.0,6246.0,6221.0/)
  radii(11:20)=(/6196.0,6171.0,6146.0,6121.0,6096.0,6071.0,6046.0,6021.0,5996.0,5976.0/)
  radii(21:30)=(/5957.0,5955.0,5946.0,5921.0,5896.0,5871.0,5846.0,5821.0,5796.0,5771.0/)
  radii(31:40)=(/5746.0,5726.0,5717.0,5715.0,5696.0,5671.0,5646.0,5621.0,5596.0,5571.0/)
  radii(41:50)=(/5546.0,5521.0,5496.0,5471.0,5446.0,5421.0,5396.0,5371.0,5346.0,5321.0/)
  radii(51:60)=(/5296.0,5271.0,5246.0,5221.0,5208.5,5196.0,5183.5,5171.0,5158.5,5146.0/)
  radii(61:70)=(/5133.5,5121.0,5108.5,5096.0,5083.5,5071.0,5058.5,5046.0,5033.5,5021.0/)
  radii(71:80)=(/5008.5,4996.0,4983.5,4971.0,4958.5,4946.0,4933.5,4921.0,4908.5,4896.0/)
  radii(81:90)=(/4883.5,4871.0,4858.5,4846.0,4833.5,4821.0,4808.5,4796.0,4783.5,4771.0/)
  radii(91:100)=(/4758.5,4746.0,4733.5,4721.0,4708.5,4696.0,4683.5,4671.0,4658.5,4646.0/)
  radii(101:110)=(/4633.5,4621.0,4608.5,4596.0,4583.5,4571.0,4558.5,4546.0,4533.5,4521.0/)
  radii(111:120)=(/4508.5,4496.0,4483.5,4471.0,4458.5,4446.0,4433.5,4421.0,4408.5,4396.0/)
  radii(121:130)=(/4383.5,4371.0,4358.5,4346.0,4333.5,4321.0,4308.5,4296.0,4283.5,4271.0/)
  radii(131:140)=(/4258.5,4246.0,4233.5,4221.0,4208.5,4196.0,4183.5,4171.0,4158.5,4146.0/)
  radii(141:150)=(/4133.5,4121.0,4108.5,4096.0,4083.5,4071.0,4058.5,4046.0,4033.5,4021.0/)
  radii(151:160)=(/4008.5,3996.0,3983.5,3971.0,3958.5,3946.0,3933.5,3921.0,3908.5,3896.0/)
  radii(161:170)=(/3883.5,3871.0,3858.5,3846.0,3833.5,3821.0,3808.5,3796.0,3783.5,3771.0/)
  radii(171:180)=(/3758.5,3746.0,3733.5,3721.0,3708.5,3696.0,3683.5,3671.0,3658.5,3646.0/)
  radii(181:190)=(/3633.5,3621.0,3608.5,3596.0,3583.5,3571.0,3559.6,3548.2,3536.9,3525.5/)
  radii(191:200)=(/3514.1,3502.8,3491.4,3480.0,3479.9,3460.0,3440.0,3420.0,3400.0,3375.0/)
  radii(201:210)=(/3350.0,3325.0,3300.0,3275.0,3250.0,3225.0,3200.0,3175.0,3150.0,3125.0/)
  radii(211:220)=(/3100.0,3075.0,3050.0,3025.0,3000.0,2975.0,2950.0,2925.0,2900.0,2875.0/)
  radii(221:230)=(/2850.0,2825.0,2800.0,2775.0,2750.0,2725.0,2700.0,2675.0,2650.0,2625.0/)
  radii(231:240)=(/2600.0,2575.0,2550.0,2525.0,2500.0,2475.0,2450.0,2425.0,2400.0,2375.0/)
  radii(241:250)=(/2350.0,2325.0,2300.0,2275.0,2250.0,2225.0,2200.0,2175.0,2150.0,2125.0/)
  radii(251:260)=(/2100.0,2075.0,2050.0,2025.0,2000.0,1975.0,1950.0,1925.0,1900.0,1875.0/)
  radii(261:270)=(/1850.0,1825.0,1800.0,1775.0,1750.0,1725.0,1700.0,1675.0,1650.0,1625.0/)
  radii(271:280)=(/1600.0,1575.0,1550.0,1525.0,1500.0,1475.0,1450.0,1425.0,1400.0,1375.0/)
  radii(281:290)=(/1350.0,1325.0,1300.0,1261.0,1241.5,1222.0,1221.0,1200.0,1100.0,1000.0/)
  radii(291:298)=(/ 900.0, 800.0, 700.0, 600.0, 500.0, 400.0, 300.0, 200.0/)

  svel(1:10)=(/ 3.200, 3.200, 3.650, 3.750, 4.600, 4.600, 4.600, 4.600, 4.500, 4.500/)
  svel(11:20)=(/ 4.500, 4.480, 4.480, 4.480, 4.510, 4.570, 4.630, 4.680, 4.730, 4.770/)
  svel(21:30)=(/ 4.810, 5.050, 5.070, 5.110, 5.150, 5.190, 5.250, 5.290, 5.330, 5.390/)
  svel(31:40)=(/ 5.450, 5.520, 5.620, 5.830, 5.950, 6.070, 6.140, 6.190, 6.220, 6.240/)
  svel(41:50)=(/ 6.260, 6.279, 6.296, 6.313, 6.330, 6.346, 6.363, 6.380, 6.396, 6.411/)
  svel(51:60)=(/ 6.427, 6.443, 6.458, 6.474, 6.481, 6.489, 6.497, 6.504, 6.511, 6.518/)
  svel(61:70)=(/ 6.525, 6.532, 6.539, 6.547, 6.554, 6.561, 6.568, 6.575, 6.582, 6.589/)
  svel(71:80)=(/ 6.596, 6.603, 6.610, 6.617, 6.624, 6.630, 6.637, 6.643, 6.650, 6.657/)
  svel(81:90)=(/ 6.664, 6.670, 6.677, 6.683, 6.690, 6.696, 6.703, 6.709, 6.716, 6.722/)
  svel(91:100)=(/ 6.728, 6.734, 6.741, 6.747, 6.753, 6.759, 6.766, 6.772, 6.778, 6.784/)
  svel(101:110)=(/ 6.790, 6.796, 6.803, 6.809, 6.815, 6.821, 6.827, 6.833, 6.839, 6.845/)
  svel(111:120)=(/ 6.850, 6.856, 6.862, 6.868, 6.873, 6.879, 6.885, 6.891, 6.896, 6.902/)
  svel(121:130)=(/ 6.908, 6.914, 6.919, 6.925, 6.931, 6.937, 6.942, 6.948, 6.954, 6.960/)
  svel(131:140)=(/ 6.965, 6.971, 6.977, 6.983, 6.988, 6.994, 6.999, 7.005, 7.010, 7.016/)
  svel(141:150)=(/ 7.021, 7.027, 7.032, 7.038, 7.043, 7.049, 7.055, 7.060, 7.065, 7.071/)
  svel(151:160)=(/ 7.077, 7.083, 7.089, 7.094, 7.099, 7.105, 7.110, 7.116, 7.122, 7.127/)
  svel(161:170)=(/ 7.132, 7.138, 7.143, 7.149, 7.155, 7.161, 7.166, 7.171, 7.176, 7.182/)
  svel(171:180)=(/ 7.191, 7.200, 7.222, 7.245, 7.267, 7.290, 7.287, 7.285, 7.283, 7.280/)
  svel(181:190)=(/ 7.277, 7.275, 7.273, 7.270, 7.267, 7.265, 7.264, 7.263, 7.261, 7.260/)
  svel(191:200)=(/ 7.259, 7.257, 7.256, 7.255, 8.009, 8.046, 8.082, 8.119, 8.155, 8.199/)
  svel(201:210)=(/ 8.242, 8.285, 8.328, 8.369, 8.410, 8.451, 8.492, 8.530, 8.569, 8.608/)
  svel(211:220)=(/ 8.646, 8.682, 8.718, 8.754, 8.790, 8.824, 8.858, 8.892, 8.926, 8.957/)
  svel(221:230)=(/ 8.988, 9.019, 9.050, 9.080, 9.109, 9.138, 9.168, 9.196, 9.223, 9.251/)
  svel(231:240)=(/ 9.279, 9.305, 9.331, 9.358, 9.384, 9.409, 9.434, 9.459, 9.484, 9.508/)
  svel(241:250)=(/ 9.532, 9.555, 9.579, 9.601, 9.624, 9.646, 9.669, 9.690, 9.711, 9.733/)
  svel(251:260)=(/ 9.754, 9.774, 9.794, 9.815, 9.835, 9.854, 9.873, 9.893, 9.912, 9.930/)
  svel(261:270)=(/ 9.949, 9.967, 9.986,10.003,10.021,10.038,10.056,10.073,10.090,10.106/)
  svel(271:280)=(/10.122,10.139,10.155,10.171,10.187,10.203,10.219,10.234,10.250,10.265/)
  svel(281:290)=(/10.280,10.295,10.310,10.333,10.344,10.356,11.028,11.036,11.072,11.105/)
  svel(291:298)=(/11.135,11.162,11.185,11.206,11.223,11.237,11.248,11.256/)

  IF (radius > 6371.0) THEN
    refvel = 3.200
  ELSEIF (radius < 200.0) THEN
    refvel = 11.256
  ELSE
    dif = ABS(radii - radius)
    J=MINLOC(dif)
    K=J(1)
    IF (radius >= radii(K)) THEN
      L = K-1
    ELSE
      L = K+1
    ENDIF
    m = (radii(L) - radii(K))/(svel(L) - svel(K))
    refvel = svel(K) + (radius - radii(K))/m
  ENDIF

  END FUNCTION refvel
  !============================================================================!


END MODULE tomogrand


