MODULE gcarcs
CONTAINS

  !Latitude, Longitude to Cartesian Coordinates
  !***************************************************
  SUBROUTINE cartesian(Lat, Lon, Rad, x, y, z)
  ! Takes input Latitude, Longitude, and Radius
  ! Returns x, y, z coordinates
  IMPLICIT NONE
  REAL, PARAMETER :: pi=3.141592653589793
  REAL, INTENT(IN) :: Lat, Lon, Rad
  REAL, INTENT(OUT) :: x, y, z
  REAL :: rho, phi, theta, dtr

  dtr = pi/180.0
  
  rho = Rad; theta = Lon*dtr
  Phi = (90.0 - Lat)*dtr

  x = rho*SIN(Phi)*COS(theta)
  y = rho*SIN(Phi)*SIN(theta)
  z = rho*COS(Phi)

  END SUBROUTINE cartesian

  !Cartesian Coordinates to Latitude and Longitude
  !***************************************************
  SUBROUTINE latlon(x, y, z, Lat, Lon, Rad)
  IMPLICIT NONE
  ! For input cartesian coordinates x, y, and z
  ! this subroutine converts to that points
  ! Latitude and Longitude, and also gives the
  ! Radius of the sphere where the point is
  ! defined
  REAL, PARAMETER :: pi=3.141592653589793
  REAL, INTENT(IN) :: x, y, z
  REAL, INTENT(OUT) :: Lat, Lon, Rad
  REAL :: arg, rtd, phi

  rtd = 180.0/pi
  Rad = SQRT(x**2 + y**2 + z**2)
  Lon = (ATAN2(y,x))*rtd
  arg = z/Rad
  phi = (ACOS(arg))*rtd
  Lat = 90.0 - phi 

  END SUBROUTINE latlon

  !Cross Product for determining euler rotation pole
  !Angle between two points on a sphere and
  !Distance between two points on a sphere
  !***************************************************
  SUBROUTINE cross(v1, v2, v3, w1, w2, w3, u1, u2, u3, alpha, s)
  ! For vectors v and w, where
  ! v = (v1, v2, v3)
  ! w = (w1, w2, w3)
  ! the cross product v x w = x = (x1, x2, x3)
  ! output is u = (u1, u2, u3); where u is the unit
  ! vector in the direction of the vector x
  ! the angle between v and w is given as alpha;
  ! where alpha = arcsin(|v x w|/|v||w|)
  ! s is the distance along the arc between the two
  ! endpoints of vectors v and w, where s=r*theta
  ! alpha and s are only valid for angles <= 90 deg
  IMPLICIT NONE
  REAL, PARAMETER :: pi=3.141592653589793
  REAL, INTENT(IN) :: v1, v2, v3, w1, w2, w3
  REAL, INTENT(OUT) :: u1, u2, u3, alpha, s
  REAL :: magV, magW, magX, arg, rtd, theta
  REAL :: x1, x2, x3

  rtd = 180.0/pi

  x1 = v2*w3 - v3*w2
  x2 = -v1*w3 + v3*w1
  x3 = v1*w2 - v2*w1

  magV = SQRT(v1**2 + v2**2 + v3**2)
  magW = SQRT(w1**2 + w2**2 + w3**2)
  magX = SQRT(x1**2 + x2**2 + x3**2)

  u1 = x1/magX; u2 = x2/magX; u3 = x3/magX

  arg = magX/(magV*magW)
  theta = ASIN(arg)
  s = magV*theta
  alpha = (ASIN(arg))*rtd 

  END SUBROUTINE cross

  !Dot Product between two vectors.  Extremely usefull for
  !determining the angle and distance between two points
  !on a shpere. 
  SUBROUTINE dot(v1, v2, v3, w1, w2, w3, dprod, alpha, s)
  ! For vectors v and w, where
  ! v = (v1, v2, v3)
  ! w = (w1, w2, w3)
  ! the dot product output is the scalar variable dprod
  ! the angle between the vectors is given as alpha, where
  ! alpha = arccos(v*w/|v||w|)
  ! s is the distance along the arc between the two endpoints
  ! of vectors v and w, where s=r*theta.  alpha and s are
  ! valid for angles <= 180 deg
  IMPLICIT NONE
  REAL, PARAMETER :: pi=3.141592653589793
  REAL, INTENT(IN) :: v1, v2, v3, w1, w2, w3
  REAL, INTENT(OUT) :: dprod, alpha, s
  REAL :: rtd, magW, magV, arg, theta

  rtd = 180.0/pi

  dprod = v1*w1 + v2*w2 + v3*w3

  magV = SQRT(v1**2 + v2**2 + v3**2)
  magW = SQRT(w1**2 + w2**2 + w3**2)

  arg = dprod/(magV*magW)
  theta = ACOS(arg)
  s = magV*theta
  alpha = (ACOS(arg))*rtd
  END SUBROUTINE dot

  !Euler Rotation Matrix
  !***************************************************	
  SUBROUTINE euler(xi,yi,zi,ex,ey,ez,alpha,xo,yo,zo)
  ! Input xi, yi, zi  are the cartesian coordinates
  ! of the point on the globe that you want to rotate.
  ! Input ex, ey, ez are the cartesian coordinates
  ! of the unit vector describing the euler rotation
  ! pole you wish to rotate around.
  ! Input alpha is the angle to rotate by (in deg)
  ! Output xo, yo, zo are the cartesean coordinates
  ! of the rotated point.
  ! rotations must be less than 90 deg
  IMPLICIT NONE
  REAL, PARAMETER :: pi=3.141592653589793
  REAL, INTENT(IN) :: xi, yi, zi, ex, ey, ez, alpha
  REAL, INTENT(OUT) :: xo, yo, zo
  REAL, DIMENSION(3,3) :: R
  REAL :: dtr, angle

  dtr = pi/180.0
  angle = alpha*dtr

  R(1,1) = (ex**2)*(1 - COS(angle)) + COS(angle)
  R(1,2) = (ex*ey)*(1 - COS(angle)) - ez*SIN(angle)
  R(1,3) = (ex*ez)*(1 - COS(angle)) + ey*SIN(angle)
  R(2,1) = (ex*ey)*(1 - COS(angle)) + ez*SIN(angle)
  R(2,2) = (ey**2)*(1 - COS(angle)) + COS(angle)
  R(2,3) = (ey*ez)*(1 - COS(angle)) - ex*SIN(angle)
  R(3,1) = (ex*ez)*(1 - COS(angle)) - ey*SIN(angle)
  R(3,2) = (ey*ez)*(1 - COS(angle)) + ex*SIN(angle)
  R(3,3) = (ez**2)*(1 - COS(angle)) + COS(angle)

  xo = xi*R(1,1) + yi*R(1,2) + zi*R(1,3)
  yo = xi*R(2,1) + yi*R(2,2) + zi*R(2,3)
  zo = xi*R(3,1) + yi*R(3,2) + zi*R(3,3)

  END SUBROUTINE euler

END MODULE gcarcs
