#!/bin/csh
#
#  This is an example script to process unfiltered synthetics created with 
#  SHaxi.  This script creates a gaussian source-time function and convolves
#  it with unfiltered synthetics located in 'udir'.  The period of the source
#  time function is set by 'Tdom'.  The convolution is performed in SAC
#  and is done in FFT space.  This has the advantage that a large number of
#  synthetics can be filtered quickly, but in my opinion isn't as good as
#  performing the convolution strictly in the time domain.  This script
#  also uses the TauP Toolkit to set arrival picks for seismic phases
#  list with 'phases' into the SAC header.  If you don't have TauP this is
#  only one line long and can easily be commented out.  Finally, the script
#  can also perform a cut on the SAC traces around a phase defined by the
#  Taup picks.
#
#  Michael Thorne (9/19/07)
#

set udir   = ./unfiltered         #directory containing unfiltered synthetics
set Tdom   = 5.0                  #dominant period
set stype  = gauss                #source-time function
set comp   = T                    #component suffix (e.g., T, R, Z)
set phases = S-0,Sdiff-0,ScS,sS   #seismic phases for picks (TauP notation)
set bcut   = -50                  #cut time (sec) before T0 header pick
set ecut   = 200                  #cut time (sec) after T0 header pick


cp ${udir}/*.${comp} .
set file1 = `ls *.${comp} | awk 'NR == 1 {print $1}'`

echo " "
echo "Creating source time function...."
echo " "
./sfun $file1 $stype $Tdom

echo " "
echo "Convolving traces with source time function..."
echo " "
sac << END
r alpha *.${comp}
rtrend
taper type hanning width 0.05
r more ${stype}.sac
convolve master ${stype}.sac
w over
r *.${comp} 
ch b ( DIV &1,NPTS 4 )
ch b ( MUL &1,B -1.0 )
ch b ( MUL &1,B &1,DELTA )
ch t0 ( DIV $Tdom 2.0 )
ch b ( ADD &1,B &1,t0 )
ch t0 und
w over
ch t0 ( DIV &1,NPTS 2 )
ch t0 ( MUL &1,DELTA &1,t0 )
cut 0.0 &1,t0 
r
ch t0 0.0
w over
q
END

#set traveltime picks for preses $file
echo " "
echo " "
echo "Setting travel time picks..."
echo " "
taup_setsac -evdpkm -mod prem -ph $phases *.${comp}


#cut file and write new cutfile
echo " "
echo "Cutting SAC files..."
echo " "

sac << END
r *.${comp}
cuterr FILLZ
cut ( ADD ${bcut} &1,T0 ) ( ADD ${ecut} &1,T0 )
r
synch BEGIN ON
w prepend c.
q
END



rm ${stype}.sac
